//
//  MyPlugin.m
//  Runner
//
//  Created by ansun on 2021/3/3.
//
#import <Flutter/Flutter.h>
#import <Foundation/Foundation.h>
#import "MyPlugin.h"
#import <AVFoundation/AVFoundation.h>
#import <CoreText/CoreText.h>
#import <UIKit/UIKit.h>
#include "BitMapPublic.h"
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "bridgeCPP.hpp"


#define iMyPixelColorFormat  (kCGImageByteOrder32Little | kCGImageAlphaPremultipliedFirst)

#import "Runner-Swift.h"
#import <time.h>
#include <sys/time.h>

@interface UIImage(MyBitMapClass)
+(UIImage*)BitMap2UIImage:(MyBitMapClass&)bits;
@end


extern "C" void CutByClip(BitMapPublic &src,IntRect clip,MyBitMapClass &newbits);


class LPRScore { //車牌分數，利用合併車牌分數提高權數，同時也記錄次數。 modify date 2021/11/23
public:
    float newScore; //新的的分數
    int newCount; //車牌相同的個數。
    char gotByFussy;
    char failureChar; //失敗的字母個數  modify date 2021/12/01
 
    
    LPRScore()
    {
        newScore = 0;
        newCount = 0;
        gotByFussy = 0;
        failureChar = 0;
    }
    
    LPRScore(float score0,char byFussy)
    {
        failureChar = 0;
        newScore = score0;
        if (score0 > 0) {
            newCount = 1;
        } else {
            newCount = 0;
        }
        
        gotByFussy = byFussy;
    }
    
    void operator+=(LPRScore &src)
    {
            newScore += src.newScore;
            newCount += src.newCount;
        if (src.gotByFussy) {
            this->gotByFussy = src.gotByFussy;
        }
    }
    
    float GetNewScore() //新的分數
    {
        if (newCount > 0) {
            return newScore / newCount;
        }
        return 0;
    }
    
    void operator+=(float value)
    {
        newScore += value;
        newCount++;
    }
    void operator=(float value)
    {
        newScore = value;
        if (value > 0) {
            newCount = 1;
        } else {
            newCount = 0;
        }
    }
    
    void operator=(LPRScore &src)
    {
        memcpy(this,&src,sizeof(src));
    }
    
};


class DispatchMemoryForFlutter
{
    public :

    char messageType[32];
    MyBitMapClass newBits;
    char theChar[64];
    char owner_lpr_num[128];
    int division;
    double char_height;
    LPRScore match_score; //credibility confidence

    char *remark;
    char photo_id[64];
    char pairIndex[32];
    char lpr_path[300];
    char photo_path[300];

    DispatchMemoryForFlutter(char *messageType0,
                             BitMapPublic *bits,
                             char *theChar0,
                             char *lpr_no0,
                             int division0,
                             double height0,
                             LPRScore score,
                             char *remark0,
                             char *photo_id2,
                             int pairIndex0,
                             char *lpr_path0,
                             char *photo_path0)
    {
        strcpy(lpr_path,lpr_path0);
        strcpy(photo_path,photo_path0);
        if (theChar0 == 0) {
            //("text0 == 0\n");
            theChar0 = (char*)"";
        }
        if (lpr_no0 == 0) {
            lpr_no0 = (char*)"";
        }
        if (remark0 == 0) {
            remark0 = (char*)"";
        }
        this->match_score = score;
        char_height = height0;
        if (bits) {
            IntRect clip = {0,0,bits->width,bits->height};
            CutByClip(*bits, clip, newBits);
        } else {
            newBits.buff = 0;
            newBits.needDispose = 0;
        }
        strcpy(messageType,messageType0);
        strcpy(owner_lpr_num,lpr_no0);
        strcpy(theChar,theChar0);
        division = division0;
        remark = (char*)malloc(strlen(remark0)+1);
        strcpy(remark,remark0);
        strcpy(photo_id,photo_id2);
        snprintf(pairIndex,sizeof(pairIndex), "%d",pairIndex0);
    }
    
    ~DispatchMemoryForFlutter()
    {
        free(remark);
    }
};



void SendToFlutter1(DispatchMemoryForFlutter *mem)
{
    dispatch_async(dispatch_get_main_queue(), ^{ //  進到 main ui thread 時，記憶體已經不同步
        NSMutableDictionary *params = [NSMutableDictionary dictionary];
        if (mem->newBits.buff) {
            UIImage *image = [UIImage BitMap2UIImage:mem->newBits];
            NSData *data = UIImagePNGRepresentation(image);
            params[@"png"] = data;
        }
        
        
        params[@"type"] =  [NSString stringWithFormat:@"%s",mem->messageType];
        params[@"lpr_no"] = [NSString stringWithFormat:@"%s",mem->owner_lpr_num];
        params[@"theChar"] = [NSString stringWithFormat:@"%s",mem->theChar];
        params[@"estimate_height"] = [NSNumber numberWithDouble:mem->char_height];
        char divs[32];
        snprintf(divs,sizeof(divs),"%d", mem->division);
        params[@"division"] = [NSString stringWithFormat:@"%s",divs];
        
        params[@"match_score"] = [NSNumber numberWithDouble:mem->match_score.GetNewScore()] ;
        params[@"match_count"] = [NSNumber numberWithInteger:mem->match_score.newCount] ;
        params[@"remark"] = [NSString stringWithFormat:@"%s",mem->remark];
        params[@"photo_id"] = [NSString stringWithFormat:@"%s",mem->photo_id];
        params[@"pairIndex"] = [NSString stringWithFormat:@"%s",mem->pairIndex];
        params[@"lpr_path"] = [NSString stringWithFormat:@"%s",mem->lpr_path];
        params[@"photo_path"] = [NSString stringWithFormat:@"%s",mem->photo_path];
        delete mem;
        AppDelegate *app = [AppDelegate GetAPP] ;
        
        [app.cameraViewChannel invokeMethod:@"onImageCaptured" arguments:params result:^(id  _Nullable result) {
            //(@"%@",result);
        }];
    });
}



class MyMutex
{
public:
    MyMutex(){
       pthread_mutex_init(&mutexFlag, 0);
      
    }
    ~MyMutex()
    {
        pthread_mutex_destroy(&mutexFlag);
    }
    pthread_mutex_t mutexFlag;
    void Lock()
    {
        pthread_mutex_lock(&mutexFlag);
    }
    void Unlock()
    {
        pthread_mutex_unlock(&mutexFlag);
    }
    char TryLock()
    {
        return pthread_mutex_trylock(&mutexFlag) == 0;
    }
} ;

int ocrThreading = 0;
char deactivateOCRPending = 0;

static CGSize myPreviewSize {0,0};

IntRect useClip = {0,0,1,1};
char useImage = 1;

int core_count(); //corecount



void *useOwner = 0;

uint16_t ios_DefaultDivision = 0;


@interface ImageHelper : NSObject {
    
}

/** Converts a UIImage to RGBA8 bitmap.
 @param image - a UIImage to be converted
 @return a RGBA8 bitmap, or NULL if any memory allocation issues. Cleanup memory with free() when done.
 */
+ ( char *) convertUIImageToBitmapRGBA8:(UIImage *)image  rowBytes:(int*)rowBytes; // uiimagetobitmap

/** A helper routine used to convert a RGBA8 to UIImage
 @return a new context that is owned by the caller
 */
+ (CGContextRef) newBitmapRGBA8ContextFromImage:(int)width height:(int)height buffer:(char*)bitmapData;//(CGImageRef)image;


/** Converts a RGBA8 bitmap to a UIImage.
 @param buffer - the RGBA8 unsigned char * bitmap
 @param width - the number of pixels wide
 @param height - the number of pixels tall
 @return a UIImage that is autoreleased or nil if memory allocation issues
 */
+ (UIImage *) convertBitmapRGBA8ToUIImage:(unsigned char *)buffer
                                withWidth:(int)width
                               withHeight:(int)height;

@end



@implementation ImageHelper


+ (char *) convertUIImageToBitmapRGBA8:(UIImage *) image1 rowBytes:(int*)rowBytes
{
    CGSize theSize = image1.size;
    int width = theSize.width;
    int height = theSize.height;
    int bufferLength = width * height * 4;
    char *bitmapData = (char *)malloc(bufferLength);
    CGContextRef context = [self newBitmapRGBA8ContextFromImage:width height:height buffer:bitmapData];
    if(context == 0) {
        free(bitmapData);
        return NULL;
    }
    CGRect rect = CGRectMake(0, 0, width, height);
    CGImageRef quartzImage = image1.CGImage;
    CGContextDrawImage(context,rect, quartzImage); //2021/01/20
    size_t bytesPerRow = CGBitmapContextGetBytesPerRow(context);
    CGContextRelease(context);
    *rowBytes = (int)bytesPerRow;
    return bitmapData;
}

+ (CGContextRef) newBitmapRGBA8ContextFromImage:(int )width height:(int)height buffer:(char*)bitmapData
{
    CGContextRef context = NULL;
    CGColorSpaceRef colorSpace;
    size_t bitsPerPixel = 32;
    size_t bitsPerComponent = 8;
    size_t bytesPerPixel = bitsPerPixel / bitsPerComponent;
    size_t bytesPerRow = width * bytesPerPixel;
    colorSpace = CGColorSpaceCreateDeviceRGB();
    if(!colorSpace) {
        NSLog(@"Error allocating color space RGB\n");
        return NULL;
    }
    context = CGBitmapContextCreate(bitmapData,
                                    width,
                                    height,
                                    bitsPerComponent,
                                    bytesPerRow,
                                    colorSpace,
                                    iMyPixelColorFormat);    // RGBA
    if(!context) {
        printf("error: Bitmap context not created\n");
    }
    CGColorSpaceRelease(colorSpace);
    return context;
}

+ (UIImage *) convertBitmapRGBA8ToUIImage:(unsigned char *) buffer
                                withWidth:(int) width
                               withHeight:(int) height {
    
    
    size_t bufferLength = width * height * 4;
    CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, buffer, bufferLength, NULL);
    size_t bitsPerComponent = 8;
    size_t bitsPerPixel = 32;
    size_t bytesPerRow = 4 * width;
    
    CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
    if(colorSpaceRef == NULL) {
        printf("error: allocating color space");
        CGDataProviderRelease(provider);
        return nil;
    }
    
    CGBitmapInfo bitmapInfo = iMyPixelColorFormat;
    CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;
    
    CGImageRef iref = CGImageCreate(width,
                                    height,
                                    bitsPerComponent,
                                    bitsPerPixel,
                                    bytesPerRow,
                                    colorSpaceRef,
                                    bitmapInfo,
                                    provider,    // data provider
                                    NULL,        // decode
                                    YES,            // should interpolate
                                    renderingIntent);
    
    uint32_t* pixels = (uint32_t*)malloc(bufferLength);
    
    if(pixels == NULL) {
        NSLog(@"Error: Memory not allocated for bitmap");
        CGDataProviderRelease(provider);
        CGColorSpaceRelease(colorSpaceRef);
        CGImageRelease(iref);
        return nil;
    }
    
    CGContextRef context = CGBitmapContextCreate(pixels,
                                                 width,
                                                 height,
                                                 bitsPerComponent,
                                                 bytesPerRow,
                                                 colorSpaceRef,
                                                 bitmapInfo);
    
    if(context == NULL) {
        NSLog(@"Error context not created");
        free(pixels);
        CGDataProviderRelease(provider);
        CGColorSpaceRelease(colorSpaceRef);
        CGImageRelease(iref);
        return 0;
    }
    
    UIImage *image = nil;
    if(context) {
        
        CGContextDrawImage(context, CGRectMake(0.0f, 0.0f, width, height), iref);
        if (context == 0) {
            printf("2-CGBitmapContextCreate = 0");
        }
        CGImageRef imageRef = CGBitmapContextCreateImage(context);
        
        // Support both iPad 3.2 and iPhone 4 Retina displays with the correct scale
        if([UIImage respondsToSelector:@selector(imageWithCGImage:scale:orientation:)]) {
            float scale = [[UIScreen mainScreen] scale];
            image = [UIImage imageWithCGImage:imageRef scale:scale orientation:UIImageOrientationUp];
        } else {
            image = [UIImage imageWithCGImage:imageRef];
        }
        
        CGImageRelease(imageRef);
        CGContextRelease(context);
    }
    
    CGColorSpaceRelease(colorSpaceRef);
    CGImageRelease(iref);
    CGDataProviderRelease(provider);
    
    if(pixels) {
        free(pixels);
    }
    return image;
}

@end


@implementation UIImage(MyBitMapClass)

+(UIImage *) imageFromRGBA:(char *) baseAddress bytesPerRow:(int)bytesPerRow width:(int)width height:(int)height
{
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(baseAddress, width, height, 8,
                                                 bytesPerRow, colorSpace, iMyPixelColorFormat);
    if (context == 0) {
        printf("1-CGBitmapContextCreate = 0");
    }
    
    CGImageRef quartzImage = CGBitmapContextCreateImage(context);
    
    CGContextRelease(context);
    CGColorSpaceRelease(colorSpace);
    UIImage *image = [UIImage imageWithCGImage:quartzImage];
    CGImageRelease(quartzImage);
    return (image);
}

+(UIImage*)BitMap2UIImage:(MyBitMapClass&)bits
{
    
    UIImage *result = [self imageFromRGBA:(char *)bits.buff bytesPerRow:bits.rowBytes width:bits.width height:bits.height];
    
    return result;
}

@end


@interface MyFactory : NSObject
<FlutterPlatformViewFactory>

@end

@interface MyFlutterView : NSObject<FlutterPlatformView>

@end

@implementation MyFlutterView

- (UIView*)view
{
    return (UIView*)0;
}



@end

@implementation MyFactory

/**
 * Create a `FlutterPlatformView`.
 *
 * Implemented by iOS code that expose a `UIView` for embedding in a Flutter app.
 *
 * The implementation of this method should create a new `UIView` and return it.
 *
 * @param frame The rectangle for the newly created `UIView` measured in points.
 * @param viewId A unique identifier for this `UIView`.
 * @param args Parameters for creating the `UIView` sent from the Dart side of the Flutter app.
 *   If `createArgsCodec` is not implemented, or if no creation arguments were sent from the Dart
 *   code, this will be null. Otherwise this will be the value sent from the Dart code as decoded by
 *   `createArgsCodec`.
 */
- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args
{
    MyFlutterView *factory = [[MyFlutterView alloc] init];
    return factory;
}

/**
 * Returns the `FlutterMessageCodec` for decoding the args parameter of `createWithFrame`.
 *
 * Only needs to be implemented if `createWithFrame` needs an arguments parameter.
 */
//@optional
//- (NSObject<FlutterMessageCodec>*)createArgsCodec;

@end


@interface MyPlugin()

{
    @public
    
    FlutterMethodChannel *_channel;
    MyBitMapClass *imageStandby;
}

@property BOOL captureImageForAnalize;
@property BOOL scanImage;

@property (nonatomic,retain) NSMutableDictionary *fontItems;
@property (nonatomic,retain) NSTimer *suspendTimer;
@property double lastReportTime;
@end

@interface MyNSTrheadInfo : NSObject
{
    @public
    
    //DivisionQueue *useDivision;
    char photo_feed_back_to_flutter;
}
@property int nsThreadIndex;
@property char dead;

@end


@implementation  MyNSTrheadInfo

@end

//extern char charbits_path[300];
//extern char lpr_temp_path[300];
//extern int ocrThreading;
//extern char deactivateOCRPending;

@implementation MyPlugin


-(void)SuspendThread
{
    self.suspendForUI_Slider = 1;
    if (self.suspendTimer) {
        @try {
            [self.suspendTimer invalidate];
        } @catch (NSException *e) {
            NSLog(@" [self.suspendTimer invalidate] %@",e);
        }
        
        
        // self.suspendTimer = 0;
    }
    self.suspendTimer = [ NSTimer scheduledTimerWithTimeInterval:1.5 repeats:NO block:^(NSTimer * _Nonnull timer) {
        self.suspendForUI_Slider = 0;
        //[self.suspendTimer invalidate];
        self.suspendTimer = 0;
    }
                         ] ;
}



//每分鐘統計一次，動態調整執行緒平行個數。 modify date 2021/12/05

double TimeToSeconds(timeval tp)
{
    double secs = tp.tv_sec + (double)tp.tv_usec / 1000000;
    return secs;
}

double TimeNowSeconds() //double clock0()
{
    struct timeval tp;
    gettimeofday(&tp, NULL);
    return TimeToSeconds(tp);
  
}


extern long finishedBySeconds[60];
extern int jobsPerSeconds; //限制每秒鐘執行 thread個數。
extern double multiThreadGapTime; //每秒鐘允許最多個處理照片的間隔時間。
int CheckJobCapability();



char pattern_bath[320] = {0};
char *install_key =  "20240401-09025237-00010001-debug+HEHFHCNCJFHHDPEDEKEEEAECEBEHECEDEHEBEDEDEJDPEDEKEEEAECEBEHECEDEHEBEDEFEHDPEDEGEJEDEHEEEJEAEKCMDBCMDBCMGBCQGRGGGGGKGDGKGBGN"; //offical debug

-(void)deactivateOCR
{
    //必須先關掉相機停止送照片
    deactivateOCRPending = 1;
    
    while (ocrThreading > 0) {
        usleep(500); //等候執行中的 thread 完成後停掉
    }
    apii_dispose_ocr_pattern();
  
}


-(void)activateOCR
{
    deactivateOCRPending = 0;
    
    const char *error = apii_init_ocr_pattern(pattern_bath,install_key);
    printf("pattern_bath %s\n",pattern_bath);
    printf("pattern_bath error %s\n",error);
    
    int core = core_count();
    
    useOwner = (__bridge void*)self;
    
    // core = iDivisionCount; // modify date 2021/11/11 對應到 divisionCount 以方便自動偵測手機效能，
    // 如果效能太差就停掉 dividion 0/1,並且持續增加延遲時間。 因為 iPhone 回報的 CPU core個數不精確，因為iPhone有變頻特性，也許可以支援較多 thread。
    // android 可能也有此現象。
    
    for (int i=0;i<core;i++) {
        MyNSTrheadInfo *myInfo =  [[MyNSTrheadInfo alloc] init];
        
        myInfo.nsThreadIndex = i;
        [NSThread detachNewThreadSelector:@selector(MyNSThreadForRecognition:) toTarget:self withObject:myInfo];
        ocrThreading++;
    }
    
}



-(void)setFlutterPaths:(NSDictionary *)params
{
    //NSLog(@"%@",params);
    NSString *char_path = params[@"charbits"];
    if (char_path) {
     //   init_charbits_path(char_path.UTF8String);
       // strcpy(charbits_path,char_path.UTF8String);
    }
    NSString *lpr_path = params[@"lpr_path"];
    if (lpr_path) {
     //   init_lpr_temp_path(lpr_path.UTF8String);
       // strcpy(lpr_temp_path,lpr_path.UTF8String);
    }
    NSString *pat_path = params[@"patterns"];
    if (pat_path) {
       // init_pattern_path(pat_path.UTF8String);
        strcpy((char*)pattern_bath,pat_path.UTF8String);
        
    }
}



-(void)MyNSThreadForRecognition:(MyNSTrheadInfo*)myThreadInfo
{
    if (jobsPerSeconds == 0) {
        jobsPerSeconds = core_count();
        if (jobsPerSeconds < 8) {
            jobsPerSeconds = 8; //每秒鐘最多執行次數 ，跟 thread 無關，也跟 division count 無關
        }
    }
    multiThreadGapTime = 1.0 / jobsPerSeconds; // modify date 2021/11/07
    
    while (deactivateOCRPending == 0) { //當flutter切換主畫面時，準備停掉OCR，釋放記憶體。就要停掉這些 thread。
        double exhaustTime = 0;
        
       
        MyBitMapClass * doThis_bitmap = 0;
        if (self.suspendForUI_Slider) {
            // if setSLider, suspend thread 可以讓UI操作比較順暢。
            //("suspend...\n");
        } else {
            //MyBitMapClass *bits2 = 0;
            //if (bits2) {
           //     doThis_bitmap = bits2;
           // } else {
                //double timeStart = TimeNowSeconds();
                @synchronized (self) {
                    static double timerLast1 = TimeNowSeconds();
                    double timerDelay = TimeNowSeconds() - timerLast1;
                    if (timerDelay > multiThreadGapTime) {
                        // modify date 2021/11/07 不管有幾個thread，透過這個限制每秒鐘可以執行的總個數，
                        // 確保不會在短時間內耗盡全部的thread，導致發呆
                        // 也確保 手機鏡頭轉移到不同車牌時，能夠及時抓到新車牌。
                        if (self->imageStandby) {
                            doThis_bitmap = self->imageStandby;
                            
                            self->imageStandby = 0;
                            timerLast1 = TimeNowSeconds();
                        }
                    } else {
                        usleep(1);
                    }
                }
           // }
            if (doThis_bitmap) {
                double beginPhotoTime = TimeNowSeconds(); // begin for a photo
                MyBitMapClass *work_bits = 0;
                if (myPreviewSize.width > 0) {
                    //已經裁切過。 modify date 2021/12/03
                    work_bits = doThis_bitmap;
                } else {
                    work_bits = new MyBitMapClass;
                }
                IntRect myClip = useClip;
                myClip.x = 0;
                myClip.y = 0;
                if (work_bits != doThis_bitmap) {
                    //使用 camera Layer 預覽才需要裁切。
                    CutByClip(*doThis_bitmap, myClip, *work_bits);
                    
                    delete doThis_bitmap;
                    doThis_bitmap = 0;
                }
               
                int miniCharHeightToRecognize = 12;
                void *userRef = 0;
                apii_DoOCRImage(*work_bits, myClip,  miniCharHeightToRecognize, ocr_call_back, userRef);
                
                
                delete work_bits;
            }
        }
        AppDelegate *app = [AppDelegate GetAPP];
        if ((exhaustTime > 5.0) || (app.memoryWarning > 0)) { //CPU 太慢則停止 thread,   modify date 2021/11/11
          app.memoryWarning = 0;
          //2023/02/03 針對iPhone記憶體不足閃退，增加 (memoryWarning > 0) 減少NSThread 個數。
            if (myThreadInfo.nsThreadIndex > 2) { // 最少保留兩個 thread,所以只有>2後面的才需要考慮。
                myThreadInfo.dead = 1;
                break; //終止這個 thread, 因為CPU可能太慢，不能支持太多個 thread。
            }
        }
    }
    @synchronized (self) {
        ocrThreading--;
    }
    
}

-(id)init
{
    self = [super init];

    
    return self;
}


-(void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result
{
    NSLog(@"");
}
/**
 * Sends a binary message to the Flutter side on the specified channel, expecting
 * no reply.
 *
 * @param channel The channel name.
 * @param message The message.
 */
- (void)sendOnChannel:(NSString*)channel message:(NSData* _Nullable)message
{
    NSLog(@"sendOnChannel message ");
}

/**
 * Sends a binary message to the Flutter side on the specified channel, expecting
 * an asynchronous reply.
 *
 * @param channel The channel name.
 * @param message The message.
 * @param callback A callback for receiving a reply.
 */
- (void)sendOnChannel:(NSString*)channel
              message:(NSData* _Nullable)message
          binaryReply:(FlutterBinaryReply _Nullable)callback
{
    NSLog(@"sendOnChannel message binaryReply");
}

/**
 * Registers a message handler for incoming binary messages from the Flutter side
 * on the specified channel.
 *
 * Replaces any existing handler. Use a `nil` handler for unregistering the
 * existing handler.
 *
 * @param channel The channel name.
 * @param handler The message handler.
 * @return An identifier that represents the connection that was just created to the channel.
 */
- (FlutterBinaryMessengerConnection)setMessageHandlerOnChannel:(NSString*)channel binaryMessageHandler:(FlutterBinaryMessageHandler _Nullable)handler
{
    NSLog(@"setMessageHandlerOnChannel");
    return 0;
}

/**
 * Clears out a channel's message handler if that handler is still the one that
 * was created as a result of
 * `setMessageHandlerOnChannel:binaryMessageHandler:`.
 *
 * @param connection The result from `setMessageHandlerOnChannel:binaryMessageHandler:`.
 */
- (void)cleanupConnection:(FlutterBinaryMessengerConnection)connection
{
    NSLog(@"cleanupConnection");
}



-(MyBitMapClass*)CMSampleBufferRef2BitMap:(CMSampleBufferRef)sampleBuffer
{
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    MyBitMapClass *bits = new MyBitMapClass();
    CVPixelBufferLockBaseAddress(imageBuffer, 0);
    
    // Get the number of bytes per row for the pixel buffer
    char *baseAddress = (char *)CVPixelBufferGetBaseAddress(imageBuffer);
    
    // Get the number of bytes per row for the pixel buffer
    int rowbytes = (int)CVPixelBufferGetBytesPerRow(imageBuffer);
    // Get the pixel buffer width and height
    bits->width = (int)CVPixelBufferGetWidth(imageBuffer);
    bits->height = (int)CVPixelBufferGetHeight(imageBuffer);
    bits->rowBytes = bits->width * 4;
    bits->buff = (unsigned char *)malloc(bits->rowBytes * bits->height);
    if (bits->rowBytes == rowbytes) {
        memcpy(bits->buff,baseAddress,bits->rowBytes * bits->height);
    } else {
        char *buff = (char*)bits->buff;
        for (int y=0;y<bits->height;y++) {
            memcpy(buff,baseAddress, bits->rowBytes);
            buff += bits->rowBytes;
            baseAddress += rowbytes;
        }
    }
    bits->needDispose = 1;
    
    CVPixelBufferUnlockBaseAddress(imageBuffer,0);
    return bits;
}




-(MyBitMapClass*)DoImageBuffer:(CMSampleBufferRef)sampleBuffer
                 //     OCR:(BOOL)realTimeRecognition
              imageBefore:(char)iCreateBeforeRecongnition
{
    MyBitMapClass *bits2 = 0;

    MyBitMapClass *srcBits;
    if (bits2) {
        srcBits = bits2;
    } else {
        srcBits = [self CMSampleBufferRef2BitMap:sampleBuffer];
    }
    
    static CGSize useSize = {0,0};
    if (useSize.width == 0) {
        useSize.width = 600;
        useSize.height = 600;
        dispatch_async(dispatch_get_main_queue(), ^{
            AppDelegate *app = [AppDelegate GetAPP];
            
            if (app.myPreviewImage) {
                useSize = app.myPreviewImage.bounds.size;
            } else {
                useSize = app.cameraPreview.bounds.size; //videoPreviewLayer 加在 myPreview.layer 下方。
            }
        });
        
        if (srcBits) {
            
            delete srcBits;
        }
        return 0; //還沒完成設定，暫不顯示。
    
    }
   
    
    if (srcBits->rowBytes == 0) {
       
        delete srcBits;
        return 0;
    } else {
        if (useSize.width && useSize.height) {
            IntRect cutsClip = {0,0,600,600};
            
            MyBitMapClass *newBits = newBits = new MyBitMapClass;
            if (bits2) {
                //debug 擷取畫面
                *newBits = *bits2;
                newBits->CloneBuff(); //因為要送到不同thread。
            } else {
                
                cutsClip.width = srcBits->width; //在flutter下，畫面寬度顯示的，剛好是照片寬度縮小；高度則是照片高的中間區段。
                cutsClip.height = srcBits->width / useSize.width * useSize.height;
                cutsClip.y = (srcBits->height - cutsClip.height) / 2;
                
                CutByClip(*srcBits,cutsClip,*newBits); //裁減
                
                if (myPreviewSize.width > 0) {
                    IntRect cloneClip = cutsClip;
                    cloneClip.x = 0;
                    cloneClip.y = 0;
                    
                    CutByClip(*newBits,cloneClip,*srcBits); //複製蓋掉
                }
            }
            useClip = cutsClip;
            if (newBits->rowBytes == 0) {
                
            } else {
                dispatch_async(dispatch_get_main_queue(), ^{
                    AppDelegate *app = [AppDelegate GetAPP];
                    if (app.myPreviewImage) {
                        UIImage *image = [UIImage BitMap2UIImage:*newBits];
                        app.myPreviewImage.contentMode = UIViewContentModeScaleToFill;
                        myPreviewSize = app.myPreviewImage.bounds.size;
                       
                        
                        app.myPreviewImage.image = image; //車牌辨識使用。 與 圖點辨識 不同。
                    } else {
                        
                    }
                    
                    delete newBits;
                });
                
            }
        }
    }
    AppDelegate *app = [AppDelegate GetAPP];
    if (app.myPreviewImage && (myPreviewSize.width == 0)) {
        //還沒設定完成,不能工作
        
        delete srcBits;
      
        return 0;
    }
    
    MyBitMapClass &bits = *srcBits;
    
    // iCreateBeforeRecongnition = 1; //1.螢幕顯示正常的照片 , 0. 顯示經過辨識修改的照片（例如標示車牌位置）
    char iCreateAfterRecongnition = (iCreateBeforeRecongnition == 2);
    
    char *oldPixel = 0;
    if (iCreateBeforeRecongnition == 1) {
        oldPixel = (char*)malloc(bits.rowBytes * bits.height);
        memcpy(oldPixel,bits.buff,bits.rowBytes * bits.height);
    }

    
    if (iCreateAfterRecongnition) {
        
        
    }
    if (oldPixel) {
        free(bits.buff); // may be modified by MobileRealTimeRecognition0
        bits.buff = (unsigned char*)oldPixel; //更換圖形
    }
    
    
    return srcBits;
}

// Create a UIImage from sample buffer data
-(MyBitMapClass*)imageFromSampleBuffer:(CMSampleBufferRef) sampleBuffer
                            //  OCR:(BOOL)realTimeRecognition
                      imageBefore:(char) iCreateBeforeRecongnition//MyCodeID:(int64_t*)codeResult
{
    /*
     imageFromSampleBuffer 處在不具有 autoreleasepool 的 thread 環境， 不適宜建立 UIImage
     */
    //DoSample
    static char recursive = 0;
    if (recursive) {
        return 0; //debug 用，防止 step時不斷的進來。
    }
    recursive++;
    MyBitMapClass *resultBits =  [self DoImageBuffer:sampleBuffer  imageBefore:iCreateBeforeRecongnition];
   // if (resultBits) {
   //     printf("resultBits width %d x height %d\n",resultBits->width,resultBits->height);
   // }
    recursive--;
    return resultBits;
}


// 單取一張照片 --> captureStillImageAsynchronouslyFromConnection
// 串流 --> captureOutput:didOutputSampleBuffer:
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
    
    if (deactivateOCRPending) {
        //("deactivateOCRPending\n");
        return;// 準備要停掉OCR
    }
    //("captureOutput\n");
    //取得低解析度照片
    self.scanImage = YES;
    if (self.scanImage) {
        static int busy = 0;
        if (busy == 0) {
            busy++;
            if (self.captureImageForAnalize) {
                
                //擷取照片後進入分析工具畫面
                // @autoreleasepool {
                
                MyBitMapClass* bits = [self imageFromSampleBuffer:sampleBuffer  imageBefore:1];
                self.captureImageForAnalize = NO;
                self.scanImage = NO;
                
                if (bits) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        
                        
                        delete bits;
                    });
                }
            } else {
                //OCR:YES 即時辨識 OCR:YES
                //OCR:NO 背景辨識
                
                
                
                //理論上有不同thread 進行辨識，所以一定有最少一個thread還在執行中，等待被拿走再準備下一個，不會耽擱。
                static double lastProcessTime = 0;
                double deltaProcessTime = TimeNowSeconds() - lastProcessTime;
                if (deltaProcessTime > multiThreadGapTime) {
                    lastProcessTime = TimeNowSeconds();
                    // 因為顯示與辨識需要同步，而辨識來不及，所以不需要處理太多照片，以免浪費時間拖垮 CPU. modify date 2021/12/03
                    
                    MyBitMapClass* bits = [self imageFromSampleBuffer:sampleBuffer  imageBefore:2 ];
                    //顯示用途在 imageFromSampleBuffer() 已經處理完畢，以下只做辨識用。
                    if (bits) {
                        
                        @synchronized (self) {
                            
                            //等待辨識
                            
                            if (self->imageStandby) {
                                
                                delete self->imageStandby;
                            }
                            if (bits->height < 0) {
                                printf("(bits->height < 0)\n");
                            }
                            if (bits->height > 4096) {
                                printf("(bits->height > 4096)\n");
                            }
                            self->imageStandby = bits;
                            
                            
                            
                        }
                        
                    }
                }
            }
            busy--;
        }
    }
    
}

- (NSURL *)applicationDocumentsDirectory {
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}


extern "C" void ocr_call_back(void *userRef, // 用戶自備
                                char *lpr_num_text, //辨識結果車牌文字,例如: 'ABC-1234'
                                float score, //分數 0.0～1.0
                                float estimate_height, // 車牌字母的高度點數pixel，在相同焦距zoom條件下，點數越大代表距離越近。
                                BitMapPublic *lpr_captured_bits // 車牌截圖
                                )
{
    
    UIImage *image = [ImageHelper convertBitmapRGBA8ToUIImage:lpr_captured_bits->buff withWidth:lpr_captured_bits->width withHeight:lpr_captured_bits->height];
    printf("%s score:%.2f 字母高度：%.2f\n",lpr_num_text,score,estimate_height);
    DispatchMemoryForFlutter *mem;
    
    mem = new DispatchMemoryForFlutter((char*)"lpr_no",
                                       lpr_captured_bits,//   lpr.lpr_bits.bits, 車牌圖形改在c++存檔，不需要傳給flutter。
                                      "",// lpr.fileName,//已經存檔的檔名傳給flutter，flutter可以讀取顯示。
                                       lpr_num_text,//  lpr.texts, //lpr
                                      0,// lpr.division_lpr,
                                       estimate_height,//  lpr.charHeight,
                                       LPRScore(score,0),
                                       //,// lpr.bestScore,
                                       "",//remark,
                                       "",//lpr.photo_id1,
                                       0,//lpr.pairIndex,
                                       "",//lpr.lpr_path,
                                       ""//lpr.photo_path_lpr
                                       );
    
    SendToFlutter1(mem);
}




-(void)setup:(double)focus expose:(double)expose zoom:(double)zoom iso:(double)iso
{
    
}

-(void)setPhotoFeedBack:(int)value
{
    //globals.photo_feedback_to_flutter = value;
}

-(double)pow:(double)value1 with:(double)value2
{
    return pow(value1,value2);
}

@end


extern "C" void SendBitmap2Save(MyBitMapClass *bits,int index)
{
    
    printf("iOS SendBitmap2NSImage \n");
}


int thread_count()
{
    return 32;
}

#import <mach/mach.h>




void ConvertBits(MyBitMapClass &src,MyBitMapClass &dst) //因為用到 NSImage 所以一定要放在 .mm 的source code
{ // ConvertBits 只有 MacOS  Desktop 版本需要支援，這裡是iOS, 假的。
    dst = src;
    dst.needDispose = 0;
}


void CheckBitmap(MyBitMapClass &bits)
{
    UIImage *image = [ImageHelper convertBitmapRGBA8ToUIImage:bits.buff withWidth:bits.width withHeight:bits.height];
    printf("image\n");
}




MyBitMapClass Bitmap2Bitmap(MyBitMapClass &bits)
{
    //NSImage *image = [NSImage ConvertBitmapToNSImage:&bits];
    //[image SaveToFile:@"/users/ansun/desktop/1.jpg"];
    //return [image NSImage2Bitmap];
    return bits;
}


void MyTerminal(char*)
{
    
}





extern "C" void CutByClip(BitMapPublic &src,IntRect clip,MyBitMapClass &newbits)
{
    if (newbits.needDispose) {
        free(newbits.buff);
    }
    newbits.width = clip.width;
    newbits.height = clip.height;
    newbits.rowBytes = newbits.width * 4;
    newbits.buff = (unsigned char *)malloc(newbits.rowBytes * newbits.height);
    newbits.needDispose = 1;
    unsigned char *srcBuff = src.buff + (clip.y ) * src.rowBytes + clip.x * 4;
    unsigned char *dstBuff = newbits.buff ;
    for (int y=0;y<clip.height;y++) {
        
        memcpy(dstBuff,srcBuff,newbits.rowBytes);
        srcBuff += src.rowBytes;
        dstBuff += newbits.rowBytes;
    }
}

typedef struct
{
    //value: aarrggbb --> memory: bb,gg,rr,aa
#ifdef __APPLE__
    
    unsigned char b,g,r,a;
#else
    //unsigned char a,r,g,b;
    unsigned char b,g,r,a;
#endif
    
} MyRGBa ;
class  MyRGB : public MyRGBa
{
public:
    uint8_t iGray()
    {
        if (0) {
            return  (r*38 + g*75 + b*15) >> 7; //加速 二進位 簡化法 在 沒有浮點運算的單晶片上比較快
        } else {
            return 0.299*r + 0.587*g + 0.114*b; //有浮點運算的CPU 比較快。 不考慮 alpha channel。
        }
    }

};

void MyBitMapClass::DumpBitMap(const char *hint ,FILE *fref )
{
    MyBitMapClass &bits = *this;
    static MyMutex mutex;
    mutex.Lock();
    FILE *out = stdout;
    if (fref) {
        out = fref;
    }
    if (hint) {
        fprintf(out,"%s ",hint);
    }
    fprintf(out,"DumpBitMap1 %d x %d\n",bits.width,bits.height);
    for (int y=0;y<bits.height;y++) {
        MyRGB *buff2 = (MyRGB *)(bits.buff + bits.rowBytes * y);
        for (int x=0;x<bits.width;x++) {
            
            int gray0 = buff2->iGray();
            if (0) {
                if (gray0 >= 0xfe) {
                    //gray0 = 0;
                    printf("_ ");
                } else {
                    printf("X ");
                }
            } else {
                //printf("%2x ",gray0);
                fprintf(out,"%3d\t",gray0);
            }
            buff2++;
        }
        fprintf(out,"\n");
    }
    mutex.Unlock();
}

MyBitMapClass::MyBitMapClass() {
    buff = 0;
    rowBytes = 0;
    width = 0;
    height = 0;
    needDispose = 0;
    
}

MyBitMapClass::MyBitMapClass(void *buff0,int rowByte0,int width0,int height0,char need0) {
    buff = (unsigned char *)buff0;
    rowBytes = rowByte0;
    width = width0;
    height = height0;
    needDispose = need0;
    
    
}

void MyBitMapClass::CloneBuff()
{
    //不理會舊 buff.
    int theSize = rowBytes * height;
    unsigned char *newBuff = (unsigned char *)malloc(theSize);
    memcpy(newBuff,buff,theSize);
    buff = newBuff;
    needDispose = 1;
}


MyBitMapClass::~MyBitMapClass() {
    /*
#if iDebugPeakXY == 1
    if (retainCount > 0) {
        printf("MyBitMapClass retainCount > 0 error\n");
    }
#endif
     */
    
    if (buff && needDispose) {
        
        free(buff);
        buff = 0;
        needDispose = 0;
    }
};


MyBitMapClass::MyBitMapClass(MyBitMapClass const &src)
{
    this->needDispose = 0;
    *this = *(MyBitMapClass*)&src;
    
}

void MyBitMapClass::operator=(MyBitMapClass &src )
{
    if (this != &src) {
        if (this->needDispose) {
            free(buff);
            
        }
        this->buff = src.buff;
        this->needDispose = 0;
        
        this->rowBytes = src.rowBytes;
        this->width = src.width;
        this->height = src.height;
    }
}

