#include "fontswindow.h"
#include "QPushButton"
#include "QList"
#include "QListWidget"
#include "QLabel"
#include "QImage"
#include "QFileDialog"
#include "QStandardPaths"
#include "QImageWriter"
#include "QImageReader"
#include <QPainter>
#include "QList"
#include <QStandardPaths>
#include <math.h>
#include "QCheckBox"
#include <QFontDatabase>
#include <QScrollArea>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include "FontParse.h"
#include "QMessageBox"

static char ShowEffect = 0; //顯示辨識學習效果
static int lineHeight = 30; //顯示字形行高

static int iFontSizeForLearning = 64;  //學習字母使用點陣大小


static QString LearnText = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        //"0123456789ABCDEFGHJKLMNPQRSTUVWXYZ"; // I/O 不採用，以免跟 0/1 混淆

static QString textForShowPixmap =  LearnText;
        //"YyJjWwGg-_";



static QPushButton *buttonLearn = nullptr;

// 用系統字形學習，此視窗用來挑選哪些字形要用來學習

class FontPair {
public:
    FontPair(FontPair*root,QString name) {
        next = root;
        fontName = name;
    }
    ~FontPair()
    {
        delete next;
    }
    QString fontName;
    QString style;
    QCheckBox *button;
    QLabel *label;

    FontPair *next;
};

static QLabel *totalSelected;

static QLabel **fImage;
static QLabel **fReconized;
static QScrollArea *scrollArea = nullptr;
static FontPair *fontRoot = nullptr;

static QList<QString> *oldSettings = new QList<QString>();
static QString settingPath = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation) + "/FontsForLearning.txt";



void ReadFontSetting()
{

    if (oldSettings) {
        delete oldSettings;
    }
    oldSettings = new QList<QString>();
    QFile qFile(settingPath);
    if (qFile.open(QIODevice::ReadOnly))
    {
        QDataStream in(&qFile);
        in >> *oldSettings;
        qFile.close();
    }
}

void WriteFontSetting()
{

    QFile qFile(settingPath);
    if (qFile.open(QIODevice::WriteOnly))
    {
        QDataStream out(&qFile);
        out << *oldSettings;
        qFile.close();
    }
}

char FontExist(QString name)
{
    foreach (QString item,*oldSettings) {
        if (item == name) {
            return 1;
        }
    }
    return 0;
}

void FontsWindow::stateChanged(int state)
{
    CollectFonts();
}

void FontsWindow::FindFonts() // FontsWindow for Qt UI ,WindowFonts for XCode
{

    ReadFontSetting();

    if (scrollArea == nullptr) {
        scrollArea = new QScrollArea(this);
        //scrollArea->setBackgroundRole(QPalette::Dark);
        QWidget *content = new QWidget(this);
        content->setBackgroundRole(QPalette::Dark);

        QFontDatabase qf;
        QStringList fonts = qf.families();
        int left = 0;

        int lineIndex = 0;
        QString myText = "0123456789ABCDEFGHJKWYZ"; //Qt UI

        foreach (QString fontName, fonts) {
            QStringList	styles = qf.styles(fontName);
            foreach (QString style,styles) {
                //if (styles.contains("Plain"))
                QString existChecked = fontName + ".." + style;
                char checked = FontExist(existChecked);
                QFont qfont	= qf.font(fontName,style, (int)(lineHeight*0.85)) ;
                fontRoot = new FontPair(fontRoot,fontName);
                fontRoot->button = new QCheckBox(content);
                fontRoot->button->setCheckState(checked?Qt::CheckState::Checked:Qt::CheckState::Unchecked);

                connect(fontRoot->button, SIGNAL (stateChanged(int)),this, SLOT (stateChanged(int)));
                fontRoot->button->resize(30,30);
                int top = lineIndex * lineHeight;
                fontRoot->button->move(left,top );
                QLabel *label = new QLabel(content);
                fontRoot->label = label;
                fontRoot->style = style;
                label->resize(this->width()-80,lineHeight);
                label->move(left + lineHeight,top);
                label->setText(fontName +"."+style+"."+myText);
                label->setFont(qfont);


                lineIndex++;
            }
        }
        scrollArea->setWidgetResizable(false);

        content->resize(this->width(),(lineIndex+1) * lineHeight);
        scrollArea->setWidget(content);

        scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOn);




    }
    int contentHeight = scrollArea->widget()->height();
    scrollArea->widget()->resize(this->width(),contentHeight);
    int width = this->width();


    int height = this->height() - (30 + (iFontSizeForLearning  + 50)*ShowEffect);
    scrollArea->resize(width,height);
    scrollArea->move(0, (30 + (iFontSizeForLearning  + 50)*ShowEffect));

    FontPair *root = fontRoot;
    while (root) {
        root->label->resize(width-lineHeight,lineHeight);
        root = root->next;
    }

    CollectFonts();

}


void FontsWindow::ShowCharPixel(LearnPackageRef learnPackage, QFont font,QChar theChar,int index)
{

    QFontMetrics metrics(font);
    QRect rect = metrics.boundingRect(theChar);
    int width = rect.width();
    int height = rect.height();

    QPixmap *srcPix = new QPixmap(width,height);
    QPainter *paint = new QPainter(srcPix);
    paint->setFont(font);

    int x = -rect.x();
    int y = -rect.y();
    QRect pixelRect = {0,0,width,height};
    paint->eraseRect(pixelRect);
    paint->drawText(x,y,theChar);
    fImage[index]->setPixmap(*srcPix);
    fImage[index]->repaint();


    QImage src(srcPix->toImage().convertToFormat(QImage::Format_ARGB32));

    char *buffer = (char*)src.bits();
    MyBitMap bits;
    bits.buff = (unsigned char*)buffer;
    bits.rowBytes = src.bytesPerLine();
    bits.width = width;
    bits.height = height;
    bits.needDispose = 0;
    FloatRect clip = {0,0,(float)width,(float)height};


    char asc = theChar.toLatin1();
   // MyInsert1Char(learnPackage,asc,bits,clip,deep,ratio_w);
    CharMaps maps;
    DoOCRFind1Char(learnPackage, maps, bits,  clip, 0);

    theChar = maps.blocks->sortedBuff[0];
    fReconized[index]->setText(theChar);

    delete paint;
    delete srcPix;

}

void FontsWindow::ShowTextPixel(LearnPackageRef learnPackage)
{
    QString names = (*oldSettings)[0];
    QStringList items = names.split("..");
    if (items.count() == 2) {
        QString fontName = items[0];
        QString fontStyle = items[1];
        QFontDatabase qf;
        QFont font = qf.font(fontName,fontStyle,iFontSizeForLearning);
        int index = 0;
        foreach (QChar theChar,textForShowPixmap) {
            ShowCharPixel(learnPackage,font, theChar,index++);
        }
    }
}


void LearnCharPixel(LearnPackageRef learnPackage, QFont font,QChar theChar,float ratio_w)
{

    QFontMetrics metrics(font);
    QRect rect = metrics.boundingRect(theChar);
    int width = rect.width();
    int height = rect.height();

    QPixmap *srcPix = new QPixmap(width,height);
    QPainter *paint = new QPainter(srcPix);
    paint->setFont(font);

    int x = -rect.x();
    int y = -rect.y();
    QRect pixelRect = {0,0,width,height};
    paint->eraseRect(pixelRect);
    paint->drawText(x,y,theChar);

    QImage src(srcPix->toImage().convertToFormat(QImage::Format_ARGB32));


    char *buffer = (char*)src.bits();
    MyBitMap bits;
    bits.buff = (unsigned char*)buffer;
    bits.rowBytes = src.bytesPerLine();
    bits.width = width;
    bits.height = height;
    bits.needDispose = 0;
    FloatRect clip = {0,0,(float)width,(float)height};

    short deep = 0;
    char asc = theChar.toLatin1();
    MyInsert1Char(learnPackage,asc,bits,clip,deep,ratio_w);


    delete paint;
    delete srcPix;

}

void FontsWindow::CollectFonts()
{
    if (oldSettings) {
        delete oldSettings;
    }
    oldSettings = new QList<QString>();
    FontPair *root = fontRoot;

    while (root) {
        Qt::CheckState status = root->button->checkState();
        if (status == Qt::CheckState::Checked) {
            oldSettings->append(root->fontName+".."+root->style);

        }
        root = root->next;
    }
    int count = oldSettings->count();
    QString text = QString::number(count);
    totalSelected->setText(text);

}


void LearnCharsBy1Font(LearnPackageRef learnPackage, QString fontName,QString fontStyle)
{
    QFontDatabase qf;
    QFont font = qf.font(fontName,fontStyle,iFontSizeForLearning);
    float ratio_w = 0;
    QFontMetrics metrics(font);
    QRect rect = metrics. boundingRect('W');
    float width = rect.width();
    float height = rect.height();
    ratio_w = width / height;
    foreach (QChar theChar,LearnText) {
        LearnCharPixel( learnPackage, font,theChar,ratio_w);
    }
}

void FontsWindow::LearnCharsByAllFonts()
{
    CollectFonts();
    if (font_learnedPattern) {
        DeleteLearningPackage(font_learnedPattern);
    }
    if (oldSettings->count() > 0) {
        char empty = 0;
        font_learnedPattern = NewLearningPackage(&empty,const char *path,false,iUseSystem_font);
        for (int i=0;i<oldSettings->count();i++)
        {
            QString names = (*oldSettings)[i];
            QStringList items = names.split("..");
            if (items.count() == 2) {
                QString fontName = items[0];
                QString fontStyle = items[1];
                LearnCharsBy1Font(font_learnedPattern,fontName,fontStyle);
            }
        }
    } else {

    }

}

void FontsWindow::SetupShowEffect()
{

    int count = textForShowPixmap.length();
    fImage = new QLabel*[count];
    fReconized = new QLabel*[count];
    for (int i=0;i<count;i++) {
        fReconized[i] = new QLabel(this);
        fReconized[i]->resize(40,20);
        fReconized[i]->move((iFontSizeForLearning + 5) * i , 30);


        fImage[i] = new QLabel(this);
        fImage[i]->resize(iFontSizeForLearning,iFontSizeForLearning);
        fImage[i]->move((iFontSizeForLearning + 5) * i , 50);


    }
}

FontsWindow::FontsWindow(QWidget *parent) : QWidget(parent)
{
    this->setWindowTitle("學習字形視窗");
    if (ShowEffect) {
        SetupShowEffect();
    }
    QLabel *hint = new QLabel(this);
    hint->setText("勾選以下字形後再點「學習」按鈕");
    hint->move(160,5);

    totalSelected = new QLabel(this);
    totalSelected->setText("-1");
    totalSelected->resize(60,20);
    totalSelected->move(80,5);

    FindFonts();

}

void FontsWindow::ButtonLearnClicked()
{


    LearnCharsByAllFonts();
    WriteFontSetting();

    if ((oldSettings->length() > 0) && ShowEffect) {
        ShowTextPixel(font_learnedPattern);
    } else {

    }

    if (oldSettings->length() == 0) {
        QMessageBox msgBox;

        msgBox.setText("勾選字形後再點「學習」按鈕");
        msgBox.setStandardButtons(QMessageBox::Ok);

        if(msgBox.exec() == QMessageBox::Ok){
            // do something
        }else {
            // do something else
        }

    }
}

void FontsWindow::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);

    if (buttonLearn == 0) {
        buttonLearn = new QPushButton(this);
        buttonLearn->setText("學習");
        connect(buttonLearn, SIGNAL (clicked()),this, SLOT (ButtonLearnClicked()));
    }
    buttonLearn->resize(70,30);
    buttonLearn->move(5,5);


    FindFonts();
}


