#include "mainTraceEdgeWindow.h"
#include "ui_mainwindow.h"
#include "QPushButton"
#include "QList"
#include "QListWidget"
#include "QLabel"
#include "QImage"
#include "QFileDialog"
#include "QStandardPaths"
#include "QImageWriter"
#include "QImageReader"
#include "QMessageBox"
#include "QList"
#include <QStandardPaths>
#include <math.h>
#include "FontParse.h"
#include "fontswindow.h"
#include "QPainter"


int iPartsSize = 64;
int iPartsWidth = 400;

#define sliderCount  5
ClickableLabel::ClickableLabel(QWidget* parent, Qt::WindowFlags f)
    : QFrame(parent,f) ,back(this), label(&back)
{

}

ClickableLabel::~ClickableLabel()
{

}

void ClickableLabel::mousePressEvent(QMouseEvent* event)
{
    emit lclicked(itemIndex);
}

static QString titles[] = {"敏感度","Blue","Green","Red","Gray"};
static MySlider *sliders[sliderCount] = {nullptr};

static QLabel *labels[sliderCount] = {nullptr};
//static QLabel *srcImage = nullptr;
static ClickableLabel *dstImage[2] = {nullptr};
static QPixmap *srcPix = nullptr;
static QPixmap *dstPix[2] = {nullptr};
static QPushButton *buttonOpen = nullptr;
static QPushButton *buttonSave = nullptr;

static int usePix = -1;
static QString srcImagePath = "";


typedef struct RgbColor
{
    unsigned char b,g,r,a;


} RgbColor;

typedef struct HsvColor
{
    unsigned char h;
    unsigned char s;
    unsigned char v;
} HsvColor;


RgbColor HsvToRgb(HsvColor hsv)
{
    RgbColor rgb;
    unsigned char region, remainder, p, q, t;

    if (hsv.s == 0)
    {
        rgb.r = hsv.v;
        rgb.g = hsv.v;
        rgb.b = hsv.v;
        return rgb;
    }

    region = hsv.h / 43;
    remainder = (hsv.h - (region * 43)) * 6;

    p = (hsv.v * (255 - hsv.s)) >> 8;
    q = (hsv.v * (255 - ((hsv.s * remainder) >> 8))) >> 8;
    t = (hsv.v * (255 - ((hsv.s * (255 - remainder)) >> 8))) >> 8;

    switch (region)
    {
    case 0:
        rgb.r = hsv.v; rgb.g = t; rgb.b = p;
        break;
    case 1:
        rgb.r = q; rgb.g = hsv.v; rgb.b = p;
        break;
    case 2:
        rgb.r = p; rgb.g = hsv.v; rgb.b = t;
        break;
    case 3:
        rgb.r = p; rgb.g = q; rgb.b = hsv.v;
        break;
    case 4:
        rgb.r = t; rgb.g = p; rgb.b = hsv.v;
        break;
    default:
        rgb.r = hsv.v; rgb.g = p; rgb.b = q;
        break;
    }

    return rgb;
}

HsvColor RgbToHsv(RgbColor rgb)
{
    HsvColor hsv;
    unsigned char rgbMin, rgbMax;

    rgbMin = rgb.r < rgb.g ? (rgb.r < rgb.b ? rgb.r : rgb.b) : (rgb.g < rgb.b ? rgb.g : rgb.b);
    rgbMax = rgb.r > rgb.g ? (rgb.r > rgb.b ? rgb.r : rgb.b) : (rgb.g > rgb.b ? rgb.g : rgb.b);

    hsv.v = rgbMax;
    if (hsv.v == 0)
    {
        hsv.h = 0;
        hsv.s = 0;
        return hsv;
    }

    hsv.s = 255 * (long) (rgbMax - rgbMin) / hsv.v;
    if (hsv.s == 0)
    {
        hsv.h = 0;
        return hsv;
    }

    if (rgbMax == rgb.r)
        hsv.h = 0 + 43 * (rgb.g - rgb.b) / (rgbMax - rgbMin);
    else if (rgbMax == rgb.g)
        hsv.h = 85 + 43 * (rgb.b - rgb.r) / (rgbMax - rgbMin);
    else
        hsv.h = 171 + 43 * (rgb.r - rgb.g) / (rgbMax - rgbMin);

    return hsv;
}


void initializeImageFileDialog(QFileDialog &dialog, QFileDialog::AcceptMode acceptMode)
{
    static bool firstDialog = true;

    if (firstDialog) {
        firstDialog = false;
        const QStringList picturesLocations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
        dialog.setDirectory(picturesLocations.isEmpty() ? QDir::currentPath() : picturesLocations.last());
    }

    QStringList mimeTypeFilters;
    const QByteArrayList supportedMimeTypes = acceptMode == QFileDialog::AcceptOpen
            ? QImageReader::supportedMimeTypes() : QImageWriter::supportedMimeTypes();
    for (const QByteArray &mimeTypeName : supportedMimeTypes)
        mimeTypeFilters.append(mimeTypeName);
    mimeTypeFilters.sort();
    dialog.setMimeTypeFilters(mimeTypeFilters);
    dialog.selectMimeTypeFilter("image/bmp");
    if (acceptMode == QFileDialog::AcceptSave)
        dialog.setDefaultSuffix("jpg");
}


void MainWindow::ParseImage()
{

    QImage src(srcPix->toImage().convertToFormat(QImage::Format_ARGB32));
    char *buffer = (char*)src.bits();
    int width = src.width();
    int height = src.height();
    int rowBytes = src.bytesPerLine();

    IntRect clip = {0,0,width,height};
    MyBitMap myBits = {(unsigned char*)buffer,rowBytes, width,height,0};


    int index = 0;
    usePix = 0;
    int color_delta = sliders[0]->value() / 8;
    //int mask =  sliders[1]->value();
    unsigned char bgra[4];
    for (int i =1;i<5;i++) {
        bgra[i-1] = (unsigned char)sliders[i]->value();
    }

    MyBitMap gray;
    MyBitMap edge;
    ParseTraceEdge(myBits,  color_delta, bgra, gray, edge);
    QImage srcImage(edge.buff, edge.width,  edge.height, QImage::Format_ARGB32);
    dstPix[1]->convertFromImage(srcImage);
    QPixmap pix2 = dstPix[1]->scaled(dstImageSize,Qt::KeepAspectRatio);
    dstImage[1]->setPixmap(pix2);

    if (gray.needDispose) {
        free(gray.buff);
    }

    if (edge.needDispose) {
        free(edge.buff);
    }


}


char MainWindow::loadFile(QString filename)
{
    srcImagePath = filename;
    char done =   srcPix->load(filename);
    if (dstImage[0]) {
        dstPix[0] = srcPix;
        QPixmap pix2 = srcPix->scaled(dstImageSize,Qt::KeepAspectRatio);
        dstImage[0]->setPixmap(pix2);
    }


    ParseImage();
    MyValueChanged(sliders[0]->value());
    return done;
}


void  MainWindow::MyValueChanged(int ivalue0)
{
    ParseImage();
}

void MainWindow::ButtonSaveClicked()
{

    usePix = 1;
    QString fileName = selectedItem->text();
    fileName = fileName.insert(fileName.length()-4,"-N");
    QString path = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
    QString savePath = path + "/" + fileName;
    QFile file(savePath);
    file.open(QIODevice::WriteOnly);
    QString suffix = savePath.right(3);
    dstPix[usePix]->save(&file, suffix.toUtf8());

}

void MainWindow::ButtonOpenClicked()
{
    SelectFolder();
    //DoOpen1ImageFile();
}
void MainWindow::SelectFolder()
{
    QString path = QFileDialog::getExistingDirectory(this, ("Select Output Folder"), QDir::currentPath());
    if (path != "") {
        RemoveAll();
        LoadFolderItems(path,1);
    }
    return;
}
void MainWindow::DoOpen1ImageFile()
{
    QFileDialog dialog(this, tr("Open Folder"));
    initializeImageFileDialog(dialog, QFileDialog::AcceptOpen);
    char stop = 0;
    while (! stop) {
        switch (dialog.exec()) {
        case  QDialog::Accepted:
        {
            stop = loadFile(dialog.selectedFiles().first());
        }
            break;
        case QDialog::Rejected:
        {
            stop = 1;
        }
            break;
        }
    }
}

MainWindow::MainWindow( QWidget *parent)
    : QMainWindow(parent)
    // , ui(new Ui::MainWindow)
{
    // fontsWindow = fontsWindow0;
    this->resize(800, 700);
}

void MainWindow::RemoveAll()
{
    foreach (MyFileItem *item,listItems) {
        delete item;
    }
    listItems.clear();
    fileLists->clear();

}

void MainWindow::LoadFolderItems(QString path,char recursive)
{

    QDir directory(path);
    QStringList images = directory.entryList();
    foreach(QString filename, images) {
        QString lower = filename.toLower();
        if (lower.endsWith(".bmp")  || lower.endsWith(".jpg") || lower.endsWith(".png")) {
            MyFileItem *item = new MyFileItem();
            item->path = path;
            item->setText(filename); //這是給 QListWidget 自動讀取的
            listItems.append(item);
        } else {
            if ((filename == ".") || (filename == "..")) {

            } else {
                if (recursive) {
                    LoadFolderItems(path + "/" + filename,1);
                }
            }
        }
    }
    foreach (MyFileItem *item,listItems) {
        fileLists->addItem(item);
    }
}


void AdjustColorRGB( char * buff0 , int width,int height ,unsigned char*  rgb)
{
    //RGBAColor *color = (RGBAColor*)buff0;
    unsigned char *buff = (unsigned char *)buff0;
    RgbColor *pixel = (RgbColor *)buff0;
    for (int y=0;y<height;y++) {
        for (int x=0;x<width;x++) {
            if (pixel->r == pixel->g && pixel->g == pixel->r) {
                //gray 不改色調
                buff += 4;
            } else {
                for (int i=0;i<4;i++) {
                    switch (i) {
                    case 0: //b
                    case 1: //g
                    case 2: //r
                        // case 3: //a
                    {
                        short value = *buff ;
                        value = value + (rgb[i] - 128)*2;
                        if (value < 0) {
                            value = 0;
                        }
                        if (value > 255) {
                            value = 255;
                        }
                        *buff = value;
                        break;
                    }
                    }
                    buff++;
                }
            }
            pixel++;
        }
    }
}


void AdjustColorBright( char * buff0 , int width,int height ,unsigned char*  light)
{
    //RGBAColor *color = (RGBAColor*)buff0;
    unsigned char *buff = (unsigned char *)buff0;
    for (int y=0;y<height;y++) {
        for (int x=0;x<width;x++) {
            RgbColor *rgb = (RgbColor *)buff;
            unsigned char alpha = rgb->a;
            HsvColor hsv = RgbToHsv(*rgb);

            int value = hsv.v + light[0] - 128;
            if (value > 255) {
                value = 255;

            }
            if (value < 0) {
                value = 0;
            }
            hsv.v = value;
            *rgb = HsvToRgb(hsv);
            rgb->a = alpha;
            buff+=4;

        }
    }
}


void AdjustColorSharp( char * oldBuff , int width,int height ,unsigned char* pSharp)
{
    double sharp = *pSharp;
    sharp = (sharp - 128) / 128  * 20;
    unsigned char *newLine = (unsigned char *)malloc(height * width * 4);
    unsigned char *newBuff = newLine;
    //RGBAColor *color = (RGBAColor*)oldBuff;
    unsigned char *buff = (unsigned char *)oldBuff;
    int height_2 = height - 2;
    int width_2 = width - 2;
    int rowbytes = width * 4;
    memcpy(newLine,buff,rowbytes);
    buff += rowbytes; // skip first line
    newLine += rowbytes;
    for (int y=1;y<height_2;y++) {
        memcpy(newLine,buff,4);
        newLine += 4;
        buff += 4; // skip first pixel

        for (int x=1;x<width_2;x++) {
            for (int i=0;i<4;i++) {
                switch (i) {

                case 1: //b
                case 2: //g
                case 3: //r

                {
                    // short value = *buff ;
                    short dtValue = 0
                            - *(buff-4 - rowbytes)   - *(buff - rowbytes)    - *(buff+4 - rowbytes)
                            - *(buff-4 )              + *buff * 8            - *(buff+4 )
                            - *(buff-4 + rowbytes)   - *(buff + rowbytes)    - *(buff+4 + rowbytes);

                    dtValue /= 8;
                    dtValue *= sharp;
                    dtValue += *buff;
                    if (dtValue < 0) dtValue = 0;
                    if (dtValue > 255) dtValue = 255;
                    *newLine = dtValue;
                    break;
                }
                case 0: //a
                    *newLine = *buff;
                    break;
                }
                buff++;
                newLine++;
            }
        }
        memcpy(newLine,buff,4);
        newLine += 4;
        buff += 4; // skip last pixel
    }
    memcpy(oldBuff,newBuff,rowbytes * height);
    free(newBuff);
}

typedef void *ContourRef;

ContourRef FilterColorSharp( char * oldBuff , int width,int height)
{

    return 0;
}


typedef ContourRef (AdjustColorPixel)( char * buff0 , int width,int height );


AdjustColorPixel *filterColorFuncs[5] = {
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp
};

#define ColorFilters  filterColorFuncs

void  MySlider::MyValueChanged(int ivalue0)
{


}

void MainWindow::onListItemClicked(QListWidgetItem* item0)
{
    selectedItem = (MyFileItem*)item0;
    QString path = selectedItem->path + "/" + selectedItem->text();
    loadFile(path);
}


void MainWindow::LabelClicked(int index) //選取圖片。分色片，四色:紅綠藍灰,每個顏色分割成有色與反色兩種圖片。有八個圖片可選
{
    if (usePix >= 0) {
        dstImage[usePix]->SetColor(0); //放掉舊圖

    }
    usePix = index;
    dstImage[usePix]->SetColor(1); //點亮新圖
    //SetupRectList();

}

void MainWindow::resizeEvent(QResizeEvent* event)
{
    QMainWindow::resizeEvent(event);

    if (buttonOpen == 0) {
        buttonOpen = new QPushButton(this);
        buttonOpen->setText("圖片目錄");
        connect(buttonOpen, SIGNAL (clicked()),this, SLOT (ButtonOpenClicked()));
    }
    buttonOpen->resize(90,30);
    buttonOpen->move(5,5);

    if (buttonSave == 0) {
        buttonSave = new QPushButton(this);
        buttonSave->setText("Save");
        connect(buttonSave, SIGNAL (clicked()),this, SLOT (ButtonSaveClicked()));
    }
    buttonSave->resize(70,30);
    buttonSave->move(260,5);



    if (fileLists == 0) {
        fileLists = new QListWidget(this);



        connect(fileLists, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(onListItemClicked(QListWidgetItem*)));
    }
    int height = this->height();
    int listWidth = 200;
    fileLists->resize(listWidth,height - 40);
    fileLists->move(0, 40);

    int sliderAreaHeight = 30 * sliderCount + 50;

    int imageWidth = (this->width() - listWidth ) /2;
    int imageHeight = this->height() - sliderAreaHeight;

    if (imageWidth > imageHeight) {
        imageWidth = imageHeight;
    } else {
        imageHeight =  imageWidth;
    }

    int dstTop = 40;
    int dstLeft =  listWidth;

    if (srcPix == nullptr) {
        srcPix = new QPixmap();
    }
    int count = 2;

    if (dstImage[0] == 0) {
        for (int i=0;i<count;i++) {
            dstImage[i] = new ClickableLabel(this);
            dstImage[i]->setAlignment(Qt::AlignCenter);
            dstImage[i]->itemIndex = i;
            connect(dstImage[i], SIGNAL(lclicked(int)), this, SLOT(LabelClicked(int)));
            dstPix[i] = new QPixmap();
        }

    }

    dstImageSize = QSize(imageWidth,imageHeight);
    for (int i=0;i<count;i++) {
        int y = i / 2;
        int x = i % 2;
        dstImage[i]->resize(imageWidth,imageHeight);
        dstImage[i]->move(dstLeft + imageWidth * x,dstTop + imageHeight * y);

        QPixmap pix2 = dstPix[i]->scaled(dstImageSize,Qt::KeepAspectRatio);
        dstImage[i]->setPixmap(pix2);

    }



    int top = this->height() - sliderAreaHeight + 50;
    int left = listWidth;
    int width = this->width() - 40 - left - 50;
    for (int i=0;i<sliderCount;i++) {
        if (labels[i] == 0) {
            labels[i] = new QLabel(this);
            labels[i]->setText(titles[i]);
        }
        labels[i]->resize(50,30);
        labels[i]->move(20+left,top);
        if (sliders[i] == 0) {
            sliders[i] = new MySlider(Qt::Horizontal, this);
            sliders[i]->name = titles[i];
            sliders[i]->funcCode = i;
            sliders[i]->setRange(0, 255);
            sliders[i]->setValue(128);
            connect(sliders[i], SIGNAL(valueChanged(int)),
                   // sliders[i],
                    this,
                    SLOT(MyValueChanged(int)));
        }
        sliders[i]->resize(width,30);
        sliders[i]->move(20 + left + 50,top+8);

        top += 30;
    }

}

MainWindow::~MainWindow()
{
    //delete ui;
}


