#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "QPushButton"
#include "QList"
#include "QListWidget"
#include "QLabel"
#include "QImage"
#include "QFileDialog"
#include "QStandardPaths"
#include "QImageWriter"
#include "QImageReader"
#include "QMessageBox"
#include "QList"
#include <QStandardPaths>
#include <math.h>
#include "FontParse.h"
#include "fontswindow.h"
#include "QPainter"


int iPartsSize = 64;
int iPartsWidth = 400;
ContourRef contour_r = nullptr;

ClickableLabel::ClickableLabel(QWidget* parent, Qt::WindowFlags f)
    : QFrame(parent,f) ,back(this), label(&back)
{

}

ClickableLabel::~ClickableLabel()
{

}

void ClickableLabel::mousePressEvent(QMouseEvent* event)
{
    emit lclicked(itemIndex);
}

static QString titles[] = {"Red","Green","Blue","Bright","Sharp"};
//static MySlider *sliders[5] = {nullptr};

//static QLabel *labels[5] = {nullptr};
//static QLabel *srcImage = nullptr;
static ClickableLabel *dstImage[4 * iGrayLevel] = {nullptr};
static QPixmap *srcPix = nullptr;
static QPixmap *dstPix[4 * iGrayLevel] = {nullptr};
static QPushButton *buttonOpen = nullptr;
static QPushButton *buttonSave = nullptr;

static int usePix = -1;
static QString srcImagePath = "";


typedef struct RgbColor
{
    unsigned char b,g,r,a;


} RgbColor;

typedef struct HsvColor
{
    unsigned char h;
    unsigned char s;
    unsigned char v;
} HsvColor;


RgbColor HsvToRgb(HsvColor hsv)
{
    RgbColor rgb;
    unsigned char region, remainder, p, q, t;

    if (hsv.s == 0)
    {
        rgb.r = hsv.v;
        rgb.g = hsv.v;
        rgb.b = hsv.v;
        return rgb;
    }

    region = hsv.h / 43;
    remainder = (hsv.h - (region * 43)) * 6;

    p = (hsv.v * (255 - hsv.s)) >> 8;
    q = (hsv.v * (255 - ((hsv.s * remainder) >> 8))) >> 8;
    t = (hsv.v * (255 - ((hsv.s * (255 - remainder)) >> 8))) >> 8;

    switch (region)
    {
    case 0:
        rgb.r = hsv.v; rgb.g = t; rgb.b = p;
        break;
    case 1:
        rgb.r = q; rgb.g = hsv.v; rgb.b = p;
        break;
    case 2:
        rgb.r = p; rgb.g = hsv.v; rgb.b = t;
        break;
    case 3:
        rgb.r = p; rgb.g = q; rgb.b = hsv.v;
        break;
    case 4:
        rgb.r = t; rgb.g = p; rgb.b = hsv.v;
        break;
    default:
        rgb.r = hsv.v; rgb.g = p; rgb.b = q;
        break;
    }

    return rgb;
}

HsvColor RgbToHsv(RgbColor rgb)
{
    HsvColor hsv;
    unsigned char rgbMin, rgbMax;

    rgbMin = rgb.r < rgb.g ? (rgb.r < rgb.b ? rgb.r : rgb.b) : (rgb.g < rgb.b ? rgb.g : rgb.b);
    rgbMax = rgb.r > rgb.g ? (rgb.r > rgb.b ? rgb.r : rgb.b) : (rgb.g > rgb.b ? rgb.g : rgb.b);

    hsv.v = rgbMax;
    if (hsv.v == 0)
    {
        hsv.h = 0;
        hsv.s = 0;
        return hsv;
    }

    hsv.s = 255 * (long) (rgbMax - rgbMin) / hsv.v;
    if (hsv.s == 0)
    {
        hsv.h = 0;
        return hsv;
    }

    if (rgbMax == rgb.r)
        hsv.h = 0 + 43 * (rgb.g - rgb.b) / (rgbMax - rgbMin);
    else if (rgbMax == rgb.g)
        hsv.h = 85 + 43 * (rgb.b - rgb.r) / (rgbMax - rgbMin);
    else
        hsv.h = 171 + 43 * (rgb.r - rgb.g) / (rgbMax - rgbMin);

    return hsv;
}


void initializeImageFileDialog(QFileDialog &dialog, QFileDialog::AcceptMode acceptMode)
{
    static bool firstDialog = true;

    if (firstDialog) {
        firstDialog = false;
        const QStringList picturesLocations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
        dialog.setDirectory(picturesLocations.isEmpty() ? QDir::currentPath() : picturesLocations.last());
    }

    QStringList mimeTypeFilters;
    const QByteArrayList supportedMimeTypes = acceptMode == QFileDialog::AcceptOpen
            ? QImageReader::supportedMimeTypes() : QImageWriter::supportedMimeTypes();
    for (const QByteArray &mimeTypeName : supportedMimeTypes)
        mimeTypeFilters.append(mimeTypeName);
    mimeTypeFilters.sort();
    dialog.setMimeTypeFilters(mimeTypeFilters);
    dialog.selectMimeTypeFilter("image/bmp");
    if (acceptMode == QFileDialog::AcceptSave)
        dialog.setDefaultSuffix("jpg");
}


//extern LearnPackageRef font_learnedPattern;

void MainWindow::ParseImage()
{
    if (fontsWindow->font_learnedPattern == nullptr) {
        fontsWindow->LearnCharsByAllFonts();
    }
    if (fontsWindow->font_learnedPattern == nullptr) {
        QMessageBox msgBox;

        msgBox.setText("請先在學習字形視窗點選「學習」以後再操作");
        msgBox.setStandardButtons(QMessageBox::Ok);

        if(msgBox.exec() == QMessageBox::Ok){
            // do something
        }else {
            // do something else
        }
    } else {
        QImage src(srcPix->toImage().convertToFormat(QImage::Format_ARGB32));
        //AdjustColorPixel *filterProc = ColorFilters[this->funcCode];
        //QImage result = ConvertBmp2Bmp(image,);
        char *buffer = (char*)src.bits();
        int width = src.width();
        int height = src.height();
        int rowBytes = src.bytesPerLine();
        //ContourRef contour =  filterProc(buffer,width,height);
        IntRect clip = {0,0,width,height};
        MyBitMap myBits = {(unsigned char*)buffer,rowBytes, width,height,0};
        if (contour_r) {
            DeleteCarImageBuffer(contour_r);
            contour_r = 0;
            SetupRectList();
        }
        contour_r = ParseCarImageBuffer(fontsWindow->font_learnedPattern,myBits,clip);
        int index = 0;
        usePix = 0;

        for (int y=0;y<4;y++) {
            for (int x=0;x<iGrayLevel;x++) {
                PaintCarImageBuffer(contour_r, (unsigned char*)buffer,4*width,width, height,index);

                dstPix[index]->convertFromImage(src);
                QPixmap pix2 = dstPix[index]->scaled(dstImageSize,Qt::KeepAspectRatio);
                dstImage[index]->setPixmap(pix2);
                index++;
            }
        }

    }
}


char MainWindow::loadFile(QString filename)
{
    srcImagePath = filename;
    char done =   srcPix->load(filename);
    if (dstImage[0]) {
        QPixmap pix2 = srcPix->scaled(dstImageSize,Qt::KeepAspectRatio);
        dstImage[0]->setPixmap(pix2);
    }

    ParseImage();
    // sliders[4]->MyValueChanged(sliders[4]->value());
    return done;
}


void MainWindow::ButtonSaveClicked()
{
    //DoSave()
    if (usePix > 0) {
        if (dstPix[usePix]  && selectedItem) {

            QString fileName = selectedItem->text();
            fileName = fileName.insert(fileName.length()-4,"-N");
            QString path = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
            QString savePath = path + "/" + fileName;
            QFile file(savePath);
            file.open(QIODevice::WriteOnly);
            QString suffix = savePath.right(3);
            dstPix[usePix]->save(&file, suffix.toUtf8());
        }
    }
}

void MainWindow::ButtonOpenClicked()
{
    SelectFolder();
    //DoOpen1ImageFile();
}
void MainWindow::SelectFolder()
{
    QString path = QFileDialog::getExistingDirectory(this, ("Select Output Folder"), QDir::currentPath());
    if (path != "") {
        LoadFolderItems(path,0);
    }
    return;
}
void MainWindow::DoOpen1ImageFile()
{
    QFileDialog dialog(this, tr("Open Folder"));
    initializeImageFileDialog(dialog, QFileDialog::AcceptOpen);
    char stop = 0;
    while (! stop) {
        switch (dialog.exec()) {
        case  QDialog::Accepted:
        {
            stop = loadFile(dialog.selectedFiles().first());
        }
            break;
        case QDialog::Rejected:
        {
            stop = 1;
        }
            break;
        }
    }
}

MainWindow::MainWindow(FontsWindow *fontsWindow0, QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    fontsWindow = fontsWindow0;
    this->resize(800, 700);
}

void MainWindow::RemoveAll()
{
    foreach (MyFileItem *item,listItems) {
        delete item;
    }
    listItems.clear();
    fileLists->clear();

}

void MainWindow::LoadFolderItems(QString path,char revursive)
{
    RemoveAll();
    QDir directory(path);
    QStringList images = directory.entryList();
    foreach(QString filename, images) {
        if (filename.endsWith(".bmp")  || filename.endsWith(".JPG") /*|| filename.endsWith(".png")*/) {
            MyFileItem *item = new MyFileItem();
            item->path = path;
            item->setText(filename); //這是給 QListWidget 自動讀取的
            listItems.append(item);
        } else {
            if ((filename == ".") || (filename == "..")) {

            } else {
                if (revursive) {
                    LoadFolderItems(path + "/" + filename,1);
                }
            }
        }
    }
    foreach (MyFileItem *item,listItems) {
        fileLists->addItem(item);
    }
}


void AdjustColorRGB( char * buff0 , int width,int height ,unsigned char*  rgb)
{
    //RGBAColor *color = (RGBAColor*)buff0;
    unsigned char *buff = (unsigned char *)buff0;
    RgbColor *pixel = (RgbColor *)buff0;
    for (int y=0;y<height;y++) {
        for (int x=0;x<width;x++) {
            if (pixel->r == pixel->g && pixel->g == pixel->r) {
                //gray 不改色調
                buff += 4;
            } else {
                for (int i=0;i<4;i++) {
                    switch (i) {
                    case 0: //b
                    case 1: //g
                    case 2: //r
                        // case 3: //a
                    {
                        short value = *buff ;
                        value = value + (rgb[i] - 128)*2;
                        if (value < 0) {
                            value = 0;
                        }
                        if (value > 255) {
                            value = 255;
                        }
                        *buff = value;
                        break;
                    }
                    }
                    buff++;
                }
            }
            pixel++;
        }
    }
}


void AdjustColorBright( char * buff0 , int width,int height ,unsigned char*  light)
{
    //RGBAColor *color = (RGBAColor*)buff0;
    unsigned char *buff = (unsigned char *)buff0;
    for (int y=0;y<height;y++) {
        for (int x=0;x<width;x++) {
            RgbColor *rgb = (RgbColor *)buff;
            unsigned char alpha = rgb->a;
            HsvColor hsv = RgbToHsv(*rgb);

            int value = hsv.v + light[0] - 128;
            if (value > 255) {
                value = 255;

            }
            if (value < 0) {
                value = 0;
            }
            hsv.v = value;
            *rgb = HsvToRgb(hsv);
            rgb->a = alpha;
            buff+=4;

        }
    }
}


void AdjustColorSharp( char * oldBuff , int width,int height ,unsigned char* pSharp)
{
    double sharp = *pSharp;
    sharp = (sharp - 128) / 128  * 20;
    unsigned char *newLine = (unsigned char *)malloc(height * width * 4);
    unsigned char *newBuff = newLine;
    //RGBAColor *color = (RGBAColor*)oldBuff;
    unsigned char *buff = (unsigned char *)oldBuff;
    int height_2 = height - 2;
    int width_2 = width - 2;
    int rowbytes = width * 4;
    memcpy(newLine,buff,rowbytes);
    buff += rowbytes; // skip first line
    newLine += rowbytes;
    for (int y=1;y<height_2;y++) {
        memcpy(newLine,buff,4);
        newLine += 4;
        buff += 4; // skip first pixel

        for (int x=1;x<width_2;x++) {
            for (int i=0;i<4;i++) {
                switch (i) {

                case 1: //b
                case 2: //g
                case 3: //r

                {
                    // short value = *buff ;
                    short dtValue = 0
                            - *(buff-4 - rowbytes)   - *(buff - rowbytes)    - *(buff+4 - rowbytes)
                            - *(buff-4 )              + *buff * 8            - *(buff+4 )
                            - *(buff-4 + rowbytes)   - *(buff + rowbytes)    - *(buff+4 + rowbytes);

                    dtValue /= 8;
                    dtValue *= sharp;
                    dtValue += *buff;
                    if (dtValue < 0) dtValue = 0;
                    if (dtValue > 255) dtValue = 255;
                    *newLine = dtValue;
                    break;
                }
                case 0: //a
                    *newLine = *buff;
                    break;
                }
                buff++;
                newLine++;
            }
        }
        memcpy(newLine,buff,4);
        newLine += 4;
        buff += 4; // skip last pixel
    }
    memcpy(oldBuff,newBuff,rowbytes * height);
    free(newBuff);
}

typedef void *ContourRef;

ContourRef FilterColorSharp( char * oldBuff , int width,int height)
{

    return 0;
}


typedef ContourRef (AdjustColorPixel)( char * buff0 , int width,int height );


AdjustColorPixel *filterColorFuncs[5] = {
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp,
    FilterColorSharp
};

#define ColorFilters  filterColorFuncs

void  MySlider::MyValueChanged(int ivalue0)
{
    unsigned char ivalue = (unsigned char)ivalue0;
    QImage src(srcPix->toImage().convertToFormat(QImage::Format_ARGB32));
    AdjustColorPixel *filterProc = ColorFilters[this->funcCode];
    //QImage result = ConvertBmp2Bmp(image,);
    char *buffer = (char*)src.bits();
    int width = src.width();
    int height = src.height();

}

void MainWindow::onListItemClicked(QListWidgetItem* item0)
{
    selectedItem = (MyFileItem*)item0;
    QString path = selectedItem->path + "/" + selectedItem->text();
    loadFile(path);
}


void MainWindow::LabelClicked(int index) //選取圖片。分色片，四色:紅綠藍灰,每個顏色分割成有色與反色兩種圖片。有八個圖片可選
{
    if (usePix >= 0) {
        dstImage[usePix]->SetColor(0); //放掉舊圖

    }
    usePix = index;
    dstImage[usePix]->SetColor(1); //點亮新圖
    SetupRectList();

}

void MainWindow::resizeEvent(QResizeEvent* event)
{
    QMainWindow::resizeEvent(event);

    if (buttonOpen == 0) {
        buttonOpen = new QPushButton(this);
        buttonOpen->setText("圖片目錄");
        connect(buttonOpen, SIGNAL (clicked()),this, SLOT (ButtonOpenClicked()));
    }
    buttonOpen->resize(90,30);
    buttonOpen->move(5,5);

    if (buttonSave == 0) {
        buttonSave = new QPushButton(this);
        buttonSave->setText("Save");
        connect(buttonSave, SIGNAL (clicked()),this, SLOT (ButtonSaveClicked()));
    }
    buttonSave->resize(70,30);
    buttonSave->move(260,5);



    if (fileLists == 0) {
        fileLists = new QListWidget(this);

        //QString path = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
        //LoadFolderItems(path);


        connect(fileLists, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(onListItemClicked(QListWidgetItem*)));
    }
    int height = this->height();
    int listWidth = 200;
    fileLists->resize(listWidth,height - 40);
    fileLists->move(0, 40);
    if (partsList == nullptr) {
        partsList = new MyListView(this);
        partsList->setResizeMode(QListView::ResizeMode::Adjust);


    }

    //int partsWidth = 300;
    int partsHeight = this->height();
    partsList->resize(iPartsWidth,partsHeight-40);
    partsList->move(this->width()-iPartsWidth,0);

    int imageWidth = (this->width() - listWidth - iPartsWidth) /2;
    int imageHeight = this->height() / 4;

    if (imageWidth > imageHeight) {
        imageWidth = imageHeight;
    } else {
        imageHeight =  imageWidth;
    }

    int dstTop = 40;
    int dstLeft =  listWidth;

    if (srcPix == nullptr) {
        srcPix = new QPixmap();
    }
    int count = 4*iGrayLevel;

    if (dstImage[0] == 0) {
        for (int i=0;i<count;i++) {
            dstImage[i] = new ClickableLabel(this);
            dstImage[i]->setAlignment(Qt::AlignCenter);
            dstImage[i]->itemIndex = i;
            connect(dstImage[i], SIGNAL(lclicked(int)), this, SLOT(LabelClicked(int)));
            dstPix[i] = new QPixmap();
        }

    }

    dstImageSize = QSize(imageWidth,imageHeight);
    for (int i=0;i<count;i++) {
        int y = i / iGrayLevel;
        int x = i % iGrayLevel;
        dstImage[i]->resize(imageWidth,imageHeight);
        dstImage[i]->move(dstLeft + imageWidth * x,dstTop + imageHeight * y);

        QPixmap pix2 = dstPix[i]->scaled(dstImageSize,Qt::KeepAspectRatio);
        dstImage[i]->setPixmap(pix2);

    }

}

MainWindow::~MainWindow()
{
    delete ui;
}




MyModel::MyModel(QObject *parent) : QAbstractTableModel(parent)
{
}

int MyModel::rowCount(const QModelIndex & /*parent*/) const
{
    return itemsCount;
}

int MyModel::columnCount(const QModelIndex & /*parent*/) const
{
    return 1;
}

QVariant MyModel::data(const QModelIndex &index, int role) const
{
    if (role == Qt::DisplayRole) {

    }
    return QVariant();
}

void MainWindow::SetupRectList()
{
    if (caller == nullptr) {
        caller = new CComboBoxPopupItemDelegate(partsList);
        caller->fontsw = fontsWindow;
        caller->listw = partsList;
        partsList->setItemDelegate(caller);

        myModel = new MyModel;
        myModel->fontsw = fontsWindow;
        myModel->listw = partsList;
        partsList->setModel(myModel);

    }
    if (contour_r != nullptr) {
        char *result = GetClusterRectList(fontsWindow->font_learnedPattern, contour_r,usePix);

        caller->items = QString(result).split("\n");
        myModel->SetItemCount ( caller->items.count()-1);
    } else {
        caller->items =  QList<QString>();
        myModel->SetItemCount ( 0);
    }
    //partsList->setItemDelegate(caller);
    //partsList->setModel(myModel);

}
/*
int MyListView::sizeHintForRow(int row) const;
{
    return 48;
}
*/
QSize CComboBoxPopupItemDelegate::sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const
{
    QWidget *pat = (QWidget*)parent();
    return QSize(pat->width(), iPartsSize); //enter your values here
}
void CComboBoxPopupItemDelegate::paint(QPainter *painter,
                                       const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QFrame *frame = new QFrame(listw);
    QLabel *imageView = new QLabel(frame);


    MyBitMap bits1;
    GetBitsBlock(&bits1,contour_r,usePix,index.row(),1); //顯示用彩色
    QImage srcImage(bits1.buff, bits1.width,  bits1.height, QImage::Format_ARGB32);



    MyBitMap bits;
    GetBitsBlock(&bits,contour_r,usePix,index.row(),0); //辨識用黑白
    FloatRect clip;
    clip.origin.x = 0;
    clip.origin.y = 0;
    clip.size.width = bits.width;
    clip.size.height = bits.height;
    CharMaps result;
    DoOCRFind1Char(fontsw->font_learnedPattern, result, bits,clip,0);
    QString checkbreak = items[index.row()].mid(0,7);

    CharMaps::MatchElement *matchs = result.blocks->matchByOrders;
    int count = 0;
    QString itemsText;
    while (matchs) {

        QString line1 = QString(matchs->theChar)  +":" + QString::number((double)matchs->percentage);

        itemsText.append(line1 + ",");
        count++;
        if (count > 10) {
            break;
        }
        matchs = matchs->next2;
    }
    if (bits.needDispose) {
        free(bits.buff);
    }

    if (index.isValid()) {
        painter->save();
        QVariant var = index.data(Qt::UserRole+1);

        QRectF rect;
        rect.setX(option.rect.x());
        rect.setY(option.rect.y());
        rect.setWidth(option.rect.width()-1);
        rect.setHeight(option.rect.height()-1);

        QPainterPath path;
        path.moveTo(rect.topRight());
        path.lineTo(rect.topLeft());
        path.quadTo(rect.topLeft(), rect.topLeft());
        path.lineTo(rect.bottomLeft());
        path.quadTo(rect.bottomLeft(), rect.bottomLeft());
        path.lineTo(rect.bottomRight());
        path.quadTo(rect.bottomRight(), rect.bottomRight());
        path.lineTo(rect.topRight());
        path.quadTo(rect.topRight(), rect.topRight());

        //painter->fillRect(rect,QColor("#f0f0f0"));
        painter->setPen(QPen(QColor("#000000"))); //pen for stroke, brush for fill
        painter->setBrush(QColor("#f0f0f0")); //pen for stroke, brush for fill
        painter->drawPath(path); //pen for stroke, brush for fill
        // 鼠标悬停或者选中时改变背景色
        if (option.state.testFlag(QStyle::State_MouseOver)) {
            painter->setPen(QPen(QColor("#ebeced")));
            painter->setBrush(QColor("#ebeced"));
            painter->drawPath(path); //pen for stroke, brush for fill
        }
        if (option.state.testFlag(QStyle::State_Selected)) {
            painter->setPen(QPen(QColor("#ff0000")));
            painter->setBrush(QColor("#ffff00"));
            painter->drawPath(path);
        }


        // 图片
        QRectF iconRect = QRect(rect.left()+2, rect.top()+2, iPartsSize-4, iPartsSize-4);
        //QRectF singerRect = QRect(iconRect.right()+5, iconRect.top(), rect.width()-10-iconRect.width(), 20);
        //QRectF songNbRect = QRect(singerRect.left(), singerRect.bottom()+5, rect.width()-10-iconRect.width(), 20);

        painter->drawImage(iconRect, srcImage);
        painter->setPen(QPen(Qt::black));

        painter->setFont(QFont("Microsoft Yahei", 20));
        painter->drawText(rect.left() + iPartsSize + 5,rect.top() + iPartsSize / 2 - 5,items[index.row()] );

        painter->setPen(QPen(Qt::black));
        painter->drawText(rect.left() + iPartsSize + 5, rect.top() +  iPartsSize - 5 , itemsText);

        painter->restore();
        //  }
    }
    if (bits1.needDispose) {
        free(bits1.buff);
    }
}
