#ifndef MAINWINDOW_H
#define MAINWINDOW_H
#include "QLabel"
#include <QMainWindow>
#include "fontswindow.h"
#include "QListView"

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE



#include <QAbstractTableModel>
#include "QStyledItemDelegate"


class MyListView : public QListView
{
public:
    explicit MyListView(QWidget* parent = Q_NULLPTR) : QListView(parent) { }
    ~MyListView() override
    {

    }

/*
    virtual int sizeHintForRow(int row) const override
    {
        return 48;
    }
*/
};



class CComboBoxPopupItemDelegate : public QAbstractItemDelegate
{
public:
     CComboBoxPopupItemDelegate(QObject* parent = 0)
         : QAbstractItemDelegate(parent)
     {
     }
     void paint(QPainter *painter,
                const QStyleOptionViewItem &option, const QModelIndex &index) const override;
     virtual QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const;
     //{
     //    return QSize(60, 100); //enter your values here
    // }

     FontsWindow *fontsw;
     MyListView *listw;
     QList<QString> items;
};

class MyModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    MyModel(QObject *parent = nullptr);
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    FontsWindow *fontsw;
    int itemsCount;
    QListView *listw;

    void SetItemCount(int count) {
        beginResetModel();
        itemsCount = count;
        endResetModel();
    }
};


class ClickableLabel : public QFrame {
    Q_OBJECT

public:
    explicit ClickableLabel(QWidget* parent = Q_NULLPTR, Qt::WindowFlags f = Qt::WindowFlags());
    ~ClickableLabel();
    QFrame back;
    QLabel label;
    int itemIndex;
    void setText(QString text) {label.setText(text);};
    void resize(int width,int height) {
        QFrame::resize(width,height);
        back.resize(width-4,height-4);
        back.move(2,2);
        label.resize(width-8,height-8);
        label.move(2,2);
    }

    void setPixmap(QPixmap pix)
    {
        SetColor(0);
        label.setPixmap(pix);
    }
    void setAlignment(Qt::Alignment a)
    {
        label.setAlignment(a);
    }
    void SetColor(char color)
    {
        QPalette pal = palette();

        // set black background
        pal.setColor(QPalette::Background,Qt::black);
        this->setAutoFillBackground(true);
        this->setPalette(pal);

        pal.setColor(QPalette::Background, color?Qt::red:Qt::white);
        back.setAutoFillBackground(true);
        back.setPalette(pal);
    }
signals:
    void lclicked(int);

protected:
    virtual void mousePressEvent(QMouseEvent* event);

};


#include "QListWidgetItem"
class  MyFileItem: public QListWidgetItem
{
    public:

    QString path;

};

class  MyPartsItem: public QListWidgetItem
{
    public:

    QString path;

};

#include "QSlider"
class MySlider : public QSlider
{

    Q_OBJECT


public:
    MySlider(Qt::Orientation orientation, QWidget *parent) :  QSlider(orientation,parent){};
    QString name;
    int funcCode;

public slots:
    void MyValueChanged(int);
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    FontsWindow *fontsWindow;
    MainWindow(FontsWindow *,QWidget *parent = nullptr);
    ~MainWindow();
    void ParseImage();
    char loadFile(QString filename);
    void SetupRectList();
    void LoadFolderItems(QString path,char recursive);
    void DoOpen1ImageFile();
    void SelectFolder();
    void RemoveAll();
    QListWidget *fileLists = nullptr;
    QList <MyFileItem*> listItems;

    QSize dstImageSize;
    MyListView *partsList = nullptr;// 右側顯示分析後的小圖件
    MyFileItem *selectedItem = nullptr;
    CComboBoxPopupItemDelegate *caller = nullptr;
    MyModel *myModel = nullptr;

private slots:
    void ButtonOpenClicked();
    void ButtonSaveClicked();
    void onListItemClicked(QListWidgetItem* item);
    void LabelClicked(int);
  //  void MyValueChanged(int ivalue0);
private:
    Ui::MainWindow *ui;
    void resizeEvent(QResizeEvent *event) override;
};
#endif // MAINWINDOW_H
