//
//  ShareViewController.swift
//  ShareExtension
//
//  Created by ansun on 2022/10/05.
//

import UIKit
import Social
import MobileCoreServices


class ShareViewController: SLComposeServiceViewController {
    //let hostAppBundleIdentifier = "com.kasem.sharing"
    let sharedKey = "ShareKey"
    var sharedMedia: [SharedMediaFile] = []
    var sharedText: [String] = []
    let imageContentType = kUTTypeImage as String
    let videoContentType = kUTTypeMovie as String
    let textContentType = kUTTypeText as String
    let urlContentType = kUTTypeURL as String
    let fileURLType = kUTTypeFileURL as String;
    //let fileURLType = kUTTypeFileURL as String;
    
    override func isContentValid() -> Bool {
        // Do validation of contentText and/or NSExtensionContext attachments here
        return true
    }
    
    override func didSelectPost() {
        // This is called after the user selects Post. Do the upload of contentText and/or NSExtensionContext attachments.
        
        // Inform the host that we're done, so it un-blocks its UI. Note: Alternatively you could call super's -didSelectPost, which will similarly complete the extension context.
        //   let item  = self.extensionContext!.inputItems.first
        // let itemProvider : NSItemProvider = item.attachments.firstObject
        //    if ([itemProvider hasItemConformingToTypeIdentifier:@"public.url"]) {
        //        [itemProvider loadItemForTypeIdentifier:@"public.url"
        //                                         options:nil
        //                             completionHandler:^(NSURL *url, NSError *error) {
        // Do what you want to do with url
        //                                  [self.extensionContext completeRequestReturningItems:@[]
        //                                                                    completionHandler:nil];
        //                             }];
        //    }
        
        self.extensionContext!.completeRequest(returningItems: [], completionHandler: nil)
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        // This is called after the user selects Post. Do the upload of contentText and/or NSExtensionContext attachments.
        if let content = extensionContext!.inputItems[0] as? NSExtensionItem {
            if let contents = content.attachments {
                for (index, attachment) in (contents).enumerated() {
                    //if attachment.hasItemConformingToTypeIdentifier(imageContentType) {
                    //   handleImages(content: content, attachment: attachment, index: index)
                    //  } else
                    //  if attachment.hasItemConformingToTypeIdentifier(textContentType) {
                    //   handleText(content: content, attachment: attachment, index: index)
                    // } else
                    if attachment.hasItemConformingToTypeIdentifier(fileURLType) {
                        handleFiles(content: content, attachment: attachment, index: index)
                    }
                    //else if attachment.hasItemConformingToTypeIdentifier(urlContentType) {
                    //   handleUrl(content: content, attachment: attachment, index: index)
                    //  } else if attachment.hasItemConformingToTypeIdentifier(videoContentType) {
                    //   handleVideos(content: content, attachment: attachment, index: index)
                    //  }
                }
            }
        }
    }
    
    override func configurationItems() -> [Any]! {
        // To add configuration options via table cells at the bottom of the sheet, return an array of SLComposeSheetConfigurationItem here.
        return []
    }
    
    func getExtension(from url: URL, type: SharedMediaType) -> String {
        let parts = url.lastPathComponent.components(separatedBy: ".")
        var ex: String? = nil
        if (parts.count > 1) {
            ex = parts.last
        }
        
        if (ex == nil) {
            switch type {
            case .image:
                ex = "PNG"
            case .video:
                ex = "MP4"
            case .file:
                ex = "TXT"
            }
        }
        return ex ?? "Unknown"
    }
    
    
    func getFileName(from url: URL, type: SharedMediaType) -> String {
        var name = url.lastPathComponent
        
        if (name.isEmpty) {
            name = UUID().uuidString + "." + getExtension(from: url, type: type)
        }
        
        return name
    }
    
    func copyFile(at srcURL: URL, to dstURL: URL) -> Bool {
        do {
            if FileManager.default.fileExists(atPath: dstURL.path) {
                try FileManager.default.removeItem(at: dstURL)
            }
            try FileManager.default.copyItem(at: srcURL, to: dstURL)
        } catch (let error) {
            print("Cannot copy item at \(srcURL) to \(dstURL): \(error)")
            return false
        }
        return true
    }
    enum RedirectType {
        case media
        case text
        case file
    }
    
    private func redirectToHostApp(type: RedirectType) {
        //let infos = Bundle.main.infoDictionary! // 這裏得到的bundle是 ShareExtension ,不是 application MLPR.
        //let apiKey = Bundle.main.infoDictionary?["CFBundleURLTypes"] as? String
        
        let url = URL(string: "ShareMLPR://dataUrl=\(sharedKey)#\(type)")
        var responder = self as UIResponder?
        let selectorOpenURL = sel_registerName("openURL:")
        
        while (responder != nil) {
            if (responder?.responds(to: selectorOpenURL))! {
                let _ = responder?.perform(selectorOpenURL, with: url) //找到 app 或任何支援 openURL: 的物件
                //(responder)
            }
            responder = responder!.next
        }
        extensionContext!.completeRequest(returningItems: [], completionHandler: nil)
    }
    
    
    private func redirectToHostAppURL(path: String) {
       // let infos = Bundle.main.infoDictionary!  // 這裏得到的bundle是 ShareExtension ,不是 application MLPR.
        //let url = URL(string: path)
        //let url = URL(string: "ShareMLPR:\(path)")
        let url = URL(string: "ShareMLPR://dataUrl=#\(path)")
        var responder = self as UIResponder?
        let selectorOpenURL = sel_registerName("openURL:")
        
        while (responder != nil) {
            if (responder?.responds(to: selectorOpenURL))! {
                let _ = responder?.perform(selectorOpenURL, with: url) //找到 app 或任何支援 openURL: 的物件
            }
            responder = responder!.next
        }
        extensionContext!.completeRequest(returningItems: [], completionHandler: nil)
    }
    
    
    private func handleFiles (content: NSExtensionItem, attachment: NSItemProvider, index: Int) {
        attachment.loadItem(forTypeIdentifier: fileURLType, options: nil) { [weak self] data, error in
            
            if error == nil, let url = data as? URL, let this = self {
                
                // Always copy
                let fileName = this.getFileName(from :url, type: .file)
                let newPath = FileManager.default
                    .containerURL(forSecurityApplicationGroupIdentifier: "group.tw.com.smartinside.mlpr.query")!
                    .appendingPathComponent("Library/Caches/" + fileName)
                let copied = this.copyFile(at: url, to: newPath)
                if (copied) {
                    this.sharedMedia.append(SharedMediaFile(path: newPath.absoluteString, thumbnail: nil, duration: nil, type: .file))
                }
                
                if index == (content.attachments?.count)! - 1 {
                    let userDefaults = UserDefaults(suiteName: "group.tw.com.smartinside.mlpr.query")
                    userDefaults?.set(this.toData(data: this.sharedMedia), forKey: this.sharedKey)
                    userDefaults?.synchronize()
                   // this.redirectToHostAppURL(path: newPath.absoluteString)
                    this.redirectToHostApp(type: .file)
                }
                
            } else {
                self?.dismissWithError()
            }
        }
    }
    private func dismissWithError() {
        print("[ERROR] Error loading data!")
        let alert = UIAlertController(title: "Error", message: "Error loading data", preferredStyle: .alert)
        
        let action = UIAlertAction(title: "Error", style: .cancel) { _ in
            self.dismiss(animated: true, completion: nil)
        }
        
        alert.addAction(action)
        present(alert, animated: true, completion: nil)
        extensionContext!.completeRequest(returningItems: [], completionHandler: nil)
    }
    
    func toData(data: [SharedMediaFile]) -> Data {
        let encodedData = try? JSONEncoder().encode(data)
        return encodedData!
    }
    class SharedMediaFile: Codable {
        var path: String; // can be image, video or url path. It can also be text content
        var thumbnail: String?; // video thumbnail
        var duration: Double?; // video duration in milliseconds
        var type: SharedMediaType;
        
        
        init(path: String, thumbnail: String?, duration: Double?, type: SharedMediaType) {
            self.path = path
            self.thumbnail = thumbnail
            self.duration = duration
            self.type = type
        }
        
        // Debug method to print out SharedMediaFile details in the console
        func toString() {
            print("[SharedMediaFile] \n\tpath: \(self.path)\n\tthumbnail: \(self.thumbnail)\n\tduration: \(self.duration)\n\ttype: \(self.type)")
        }
    }
    
    enum SharedMediaType: Int, Codable {
        case image
        case video
        case file
    }
    
}
