//
//  BitMapPublic.h
//  Runner
//
//  Created by ansun on 2023/5/30.
//

#ifndef BitMapPublic_h
#define BitMapPublic_h

extern "C"  {
class BitMapPublic
{
public:
    unsigned char *buff; // 一個pixel 4個bytes： r-g-b-a
    int rowBytes;
    int width;
    int height;
};

typedef struct IntRect {
    int x;
    int y;
    int width;
    int height;
    
} IntRect;

typedef void(OCRResultCallback)(void *userRef, // 用戶自備
                                char*lpr_num_text, //辨識結果車牌文字,例如: 'ABC-1234'
                                float score, //分數 0.0～1.0
                                float estimate_height, // 車牌字母的高度點數pixel，在相同焦距zoom條件下，點數越大代表距離越近。
                                BitMapPublic *lpr_captured_bits // 車牌截圖
                                );
// apii_DoOCRImage 會自動調用二制化中間值，重複呼叫會使用不同的二制化而得到不同的辨識結果。
/*
呼叫 apii_DoOCRImage 時指定的 clip 越大則消耗記憶體越多，如果加上 multi-thread，就可能超用記憶體。
 高解析度照片會吃掉大量記憶體，而且辨識非常沒有效率。
 apii_DoOCRImage() 應該包在 nsthread 或 java thread內。
*/
void apii_DoOCRImage(
                     BitMapPublic bits_pub, //原始照片。
                     IntRect myClip, // 辨識照片中的框區範圍內，超出框外則忽略不辨識。
                     int miniCharHeightToRecognize, //辨識照片中的圖塊必須高於這個pixel 點數，越小則越慢。
                     OCRResultCallback *callback, // 回調 api，每次一個車牌號碼
                     void *userRef // 用戶自備， 用於callback 第一個參數。
);
const char *apii_init_ocr_pattern(const char *pattern_bath,const char *install_key); // 指向  packed-patttern.bin ，本來是 packed-patttern.bin.zip ，必須先解壓縮後才能呼叫此api。
void apii_dispose_ocr_pattern();

}

#endif /* BitMapPublic_h */
