//library native_camera;
//library mlpr_pkg;

import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:ocr_flutter_my/mlpr_package/main_mlpr.dart';
import 'package:photo_view/photo_view.dart';
import 'PageUIMobileCameraScanner.dart';
import 'package:http/http.dart' as http;
import 'package:http_parser/http_parser.dart';

bool cameraChannelReady = false;
MethodChannel ?channel00; // channel00 在desktop 只有一個，在 mobile 則會被設定兩次，第一次用於檢查 permission，第二次打開UI 與相機後重設為 view 通道。
MethodChannel ?channel_id;
String queryErrorMessage = "";

String patternBinPath = ""; //辨識字模樣本壓縮檔

String lpr_char_path = '';
Map<String, bool> sqlKeyBusy = {};
UploadPacked ?pendingUpload = null;
bool uploadBusy = false;
String platform0 = "";
bool runInBuildProductDebug = false;


bool doUpload = false; //開發用途，從手機傳送照片到桌機伺服器



enum CameraSessionPreset { low, medium, high, photo }
enum CameraPreviewRatio { r16_9, r11_9, r4_3, r1 }

class UploadPacked {
  UploadPacked(this.bits, this.info);

  Uint8List bits;
  String info;
}

class ImageItemDir {
  ImageItemDir(this.path) {
    List<String> names = path.split("/");
    if (names.length > 2) {
      String name1 = names[names.length - 2];
      String name2 = names[names.length - 1];
      datePath = name1 + "/" + name2;
    }
    selected = false;
  }

  String path = '';
  String datePath = '';
  bool selected = false;
}



Future<void> setSavePath(String savePath) async {
  if (Platform.isIOS) return;

  await channel00!
      .invokeMethod('setSavePath', {"savePath": savePath});
}

Future<void> setFocus(
    double x,
    double y,
    ) async {
  await channel00!
      .invokeMethod('setFocus', {"x": x, "y": y});
}

enum FlashType { auto, on, off, torch }


Future<void> setFlashType(FlashType flashType) async {
  String flashTypeString;

  switch (flashType) {
    case FlashType.auto:
      flashTypeString = "autoFlash";
      break;
    case FlashType.on:
      flashTypeString = "on";
      break;
    case FlashType.off:
      flashTypeString = "off";
      break;
    case FlashType.torch:
      flashTypeString = "torch";
      break;
  }

  await channel00!
      .invokeMethod('setFlashType', {"flashType": flashTypeString});
}


Future<String> OCRByNative(BytesBuilder ?imageData,String ?path //,String info
    ) async {
  Completer<String> completer = Completer<String>();
  Map<String, dynamic> params = {};
  if (imageData != null) {
    params["image"] = imageData
        .toBytes()
        .buffer
        .asUint8List();
  }
  if (path != null) {
    params["path"] = path;
  }
  channel00!.invokeMethod('do_ocr', params).then((text) {
    completer.complete(text);
  });
  return completer.future;
}

Future<String> ShowWebMessage(String messages) async {
  Completer<String> completer = Completer<String>();
  Map<String, dynamic> params = {};

  params["messages"] = messages;

  channel00!.invokeMethod('web_message', params).then((text) {
    completer.complete(text);
  });

  return completer.future;
}

Future<bool> DoParsePhotos(String path) async {
  return await channel00!.invokeMethod('DoParsePhotos',path);
}

Future<void> deactivateOCR() async {
  channel00!.invokeMethod('deactivateOCR').then((value) {
    debugPrint('deactivateOCR');
  //  channel00 = null;
   // camera1View = null;
  //  cameraChannelReady = false;
   // SetDesktopCallBack(); //重新設定channel00 to state with no camera
  });
}

Future<void> launchOldUI() async {
  channel00!.invokeMethod('launchOldUI');
}

Future<void> activateOCR() async {
  if (channel00 != null) {
    channel00!.invokeMethod('activateOCR').then((value) {
      debugPrint('activateOCR');
    });
  }
}

Future<String?> setMyPaths(Map<String, String> paths) async {
  debugPrint('setMyPaths channel00 = $channel00');

  Completer<String?> result = Completer<String?>();
  channel00!.invokeMethod('setPaths', paths).then(( value) {
    if (value is String) {
      result.complete(value);
    } else {
      result.complete('');
    }
  });
  return result.future;
}

Future<void> uploadFile(Uint8List data, String platform, String info) async {
//傳送檔案到桌機伺服器
  Completer completer = Completer();
  try {
    String portPhotoRecoginitonNumber = runInBuildProductDebug?"8001":"8000";
    var postUri =
    Uri.parse("http://checkin.showlendar.com:" + portPhotoRecoginitonNumber + "/upload/image");
//var postUri = Uri.parse("http://123.194.114.93:8000/upload/image");
//var postUri = Uri.parse("http://192.168.0.100:8000/upload/image");
    var request = new http.MultipartRequest("POST", postUri);
    request.fields['datetime'] = DateTime.now().toIso8601String();
    request.fields['platform'] = platform;
    request.fields["info"] = info;
    var parts1 = new http.MultipartFile.fromBytes('file1', data,
        contentType:  MediaType('image', 'jpeg'), filename: "name1");
    request.files.add(parts1);

    request.send().then((response) {
      if (response.statusCode == 200) {
        debugPrint("Uploaded!");
        completer.complete();
        debugPrint("setState 8");
//setState(() {});
      }
    });
  } catch (e) {
    debugPrint("Uploaded error ");
    debugPrint(e.toString());
    completer.completeError(e);
  }
  return completer.future;
}

void Threading() {
  if (Platform.isMacOS) {
    platform0 = "MacOS";
  } else if (Platform.isIOS) {
    platform0 = "IOS";
  } else if (Platform.isAndroid) {
    platform0 = "Android";
  } else {
    platform0 = "unknown";
  }

  if ((pendingUpload != null) && (uploadBusy == false)) {
    Uint8List bitsvalue = pendingUpload!.bits;
    String info = pendingUpload!.info;
    pendingUpload = null;
    uploadBusy = true;
    uploadFile(bitsvalue, platform0, info).then((value) {
      uploadBusy = false;
      Threading();

    });
  }
}


void UploadToServer(Uint8List bitsvalue, String info, dynamic params) {
  if (pendingUpload == null) {
    pendingUpload = new UploadPacked(bitsvalue, info);
  }
  Threading();
}

Future<void> captureImage({int maxSize=0}) async {
  await channel00!.invokeMethod('captureImage', <String, dynamic>{
    'maxSize': maxSize,
  });
}

Future<void> switchCamera() async {
  await channel00!.invokeMethod('switchCamera', null);
}

Future<void> turnOffCamera() async {
  await channel00!.invokeMethod('turnOff', null);
}

Future<void> turnOnCamera() async {
  await channel00!.invokeMethod('turnOn', null);
}

Future<void> setCameraViewSizes(double width, double height) async {
  await channel00!.invokeMethod(
      'setCameraViewSize', {"cameraWidth": width, "cameraHeight": height});
}


Future<void> setPhotoFeedBack(int value) async {
  await channel00!.invokeMethod(
      'setPhotoFeedBack', {"feedBack": value});
}


Future<String> getPublicDirectory() async {

  return await channel00!.invokeMethod(  'getPublicDirectory', {});

}


Future<bool> DoCheckForPermission() async {
//('DoCheckForPermission');
  return await channel00!.invokeMethod('checkForPermission');
}



class ImageViewer extends StatefulWidget
{

  String path = '';


  ImageViewer( this.path):super();


  @override
  _ImageViewerState createState() {
    _ImageViewerState myLineState = _ImageViewerState();
    return myLineState;
  }

}

class _ImageViewerState extends State<ImageViewer>
{
  _ImageViewerState():super() {

  }

  @override
  void dispose() {

    //ads!.dispose_ad();
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {

    return bodyImageViewer();
    /*
    return Scaffold(
      appBar: AppBar(

        title: Text("照片"),

      ),

      //body: ads?.WithGoogleAds(this, bodyImageViewer()),
      body:  bodyImageViewer(),

    );

     */
  }

  Widget bodyImageViewer()
  {
    return
      Column(
        children: [

          Expanded(
            child:
            PhotoView(
              imageProvider:  FileImage(File(widget.path))
              ,
            )
            ,
          )
        ],
      );
  }
}

