//library native_camera;
//library mlpr_pkg;

import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'PageUIMobileCameraScanner.dart';
import 'channel.dart';
import 'main_mlpr.dart';

/// Controller for a single GoogleMap instance running on the host platform.
class NativeCameraUint {
  //final MethodChannel mobile_imageCapturedChannel;

  Future<void> setSliders(Map<String, dynamic> sValues) async {
    Completer result = new Completer<void>();
    if (channel00 != null) {
      await channel00!.invokeMethod('setSliders', sValues).then((value) {
        return result.complete();
      });
    } else {
      result.complete();
    }
    return result.future;
  }

  Future getZoom() async {
    Completer result = new Completer<double>();
    channel00!.invokeMethod('getZoom').then((value) {
      if (value is double) {
        result.complete(value);
      } else {
        result.complete(0);
      }
    });
    return result.future;
  }

  NativeCameraUint._(
    //this.mobile_imageCapturedChannel,
    this._myCameraState,
  ) {
    channel00!.setMethodCallHandler(_handleImageMobileCapturedCall);
  }

  static Future<NativeCameraUint> init_channel00(
    int id,
    _MyNativeCameraState myCameraState,
  ) async {
    assert(id != null);
    channel_id = MethodChannel('plugins.flutter.io/my_camera/$id');
    debugPrint('channel_open');
    channel00 = channel_id;
    cameraChannelReady = true;

    await channel00!.invokeMethod("waitForCamera");
    debugPrint('flutter after waitForcamera');
    await activateOCR();


    return NativeCameraUint._(
      // view_channel,
      myCameraState,
    );
  }

  @visibleForTesting
  final _MyNativeCameraState _myCameraState;

  Future<dynamic> _handleImageMobileCapturedCall(MethodCall call) async {
    switch (call.method) {
      case "channel_closed":
        debugPrint('channel_closed');
        channel00 = null;
        camera1View = null;
        cameraChannelReady = false;
        SetDesktopCallBack(); //重新設定channel00 to state with no camera
        break;
      case "setZoom":
        double value = call.arguments["setZoom"];
        _myCameraState.setZoom(value);
        break;
      case "onImageCaptured": //參考：手機版本

        Uint8List? value = call.arguments["png"];
        //Image image1 = Image.memory(value);
        String lpr_no = call.arguments["lpr_no"];
        double match_score = call.arguments["match_score"];
        double estimate_height = call.arguments["estimate_height"];

        _myCameraState.onImageCaptured_mobile_1(value, lpr_no, match_score, estimate_height, '', call.arguments);
        return "no error";
        break;
      case "onFlashTypeChanged":
        //String types = call.arguments['types'] as String;
        Uint8List? image1 = null;
        _myCameraState.onImageCaptured_mobile_1(image1, "", 0, 0, "", null);
        return "no error";
        break;
      default:
        throw MissingPluginException();
        return "not implement";
    }
  }
}

enum CameraType { front, rear }

typedef Future CameraCreatedCallback(NativeCameraUint controller, double camera_width, double camera_height);
typedef void ImageCapturedCallback(Uint8List? image1, String text, double match, double estimate_height, String remark, dynamic params);
typedef void setZoomCallBack(double value);

class MyNativeCamera extends StatefulWidget {
  final CameraType initialCameraType;
  final CameraPreviewRatio cameraPreviewRatio;
  final CameraSessionPreset cameraSessionPreset;
  final CameraCreatedCallback? onCameraCreated;
  final ImageCapturedCallback? onImageCaptured_mobile_2;
  final FlashType flashType;
  final bool bestPictureSize;
  final String? fileNamePrefix;
  final Color? focusRectColor;
  final int? focusRectSize;
  final bool checkPermissionAtStartup;
  final setZoomCallBack? setZoom;


   MyNativeCamera(
      {Key? key,
      CameraType? initialCameraType,
      CameraPreviewRatio? cameraPreviewRatio,
      CameraSessionPreset? cameraSessionPreset,
      FlashType? flashType,
      bool? bestPictureSize,
      this.onCameraCreated,
      this.onImageCaptured_mobile_2,
      this.fileNamePrefix,
      this.focusRectColor,
      this.focusRectSize,
      this.checkPermissionAtStartup = true,
      this.setZoom = null})
      : this.initialCameraType = initialCameraType ?? CameraType.rear,
        this.cameraPreviewRatio = cameraPreviewRatio ?? CameraPreviewRatio.r16_9,
        this.cameraSessionPreset = cameraSessionPreset ?? CameraSessionPreset.photo,
        this.flashType = flashType ?? FlashType.auto,
        this.bestPictureSize = bestPictureSize ?? true,
        super(key: key);

  @override
  _MyNativeCameraState createState() => _MyNativeCameraState();
}

class _MyNativeCameraState extends State<MyNativeCamera> {
  Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers = null;
  CameraPreviewRatio? _cameraPreviewRatio = null;
  CameraSessionPreset? _cameraSessionPreset = null;
  FlashType? _flashType = null;
  bool hasPermission = false;
  double real_width = 0;
  double real_height = 0;
  Widget? nativeMyCameraView = null;
  bool init_ok = false;

  void my_init()
  {
    if (!init_ok) {
      init_ok = true;
      Set1channel00();

    }
  }

  @override
  void initState() {
    super.initState(); //不一定被呼叫。跟 dispose() 不是成對狀態。有時候需要配合 didUpdateWidget().
    _cameraPreviewRatio = widget.cameraPreviewRatio;
    _cameraSessionPreset = widget.cameraSessionPreset;
    _flashType = widget.flashType;

    my_init(); //不一定被呼叫。跟 dispose() 不是成對狀態。有時候需要配合 didUpdateWidget().
  }

  @override
  void dispose() {
    init_ok = false;
    deactivateOCR();
    super.dispose();
  }

  void Set1channel00() {
    debugPrint('Set1channel00');
    String previewRatio = '';
    String sessionPreset = '';
    String flashType = '';

    switch (_flashType) {
      case FlashType.on:
        flashType = "on";
        break;
      case FlashType.off:
        flashType = "off";
        break;
      case FlashType.auto:
        flashType = "autoFlash";
        break;
      case FlashType.torch:
        flashType = "torch";
        break;
    }

    switch (_cameraPreviewRatio) {
      case CameraPreviewRatio.r16_9:
        previewRatio = "16:9";
        break;
      case CameraPreviewRatio.r11_9:
        previewRatio = "11:9";
        break;
      case CameraPreviewRatio.r4_3:
        previewRatio = "4:3";
        break;
      case CameraPreviewRatio.r1:
        previewRatio = "1:1";
        break;
    }

    switch (_cameraSessionPreset) {
      case CameraSessionPreset.low:
        sessionPreset = "low";
        break;
      case CameraSessionPreset.medium:
        sessionPreset = "medium";
        break;
      case CameraSessionPreset.high:
        sessionPreset = "high";
        break;
      case CameraSessionPreset.photo:
        sessionPreset = "photo";
        break;
    }

    final Map<String, dynamic> creationParams = <String, dynamic>{
      "initialCameraType": widget.initialCameraType == CameraType.rear ? "rear" : "front",
      "previewRatio": previewRatio,
      "sessionPreset": sessionPreset,
      "flashType": flashType,
      "fileNamePrefix": widget.fileNamePrefix ?? "my_camera",
      "bestPictureSize": widget.bestPictureSize,
      "focusRectColorRed": (widget.focusRectColor != null) ? widget.focusRectColor!.red : 12,
      "focusRectColorGreen": (widget.focusRectColor != null) ? widget.focusRectColor!.green : 199,
      "focusRectColorBlue": (widget.focusRectColor != null) ? widget.focusRectColor!.blue : 12,
      "focusRectSize": widget.focusRectSize ?? 100,
      //for first run on Android (because on each device the default picture size is vary, for example MI 8 Lite's default is the lowest resolution)
    };

    if (defaultTargetPlatform == TargetPlatform.android) {
      nativeMyCameraView = AndroidView(
        viewType: 'plugins.flutter.io/my_camera',
        onPlatformViewCreated: onPlatformViewCreated,
        gestureRecognizers: gestureRecognizers,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec(),
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      nativeMyCameraView = UiKitView(
        viewType: 'plugins.flutter.io/my_camera',
        onPlatformViewCreated: onPlatformViewCreated,
        gestureRecognizers: gestureRecognizers,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec(),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    //if (!hasPermission) {
    //  return Center(child: CircularProgressIndicator());
    //}
    my_init();

    return LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
        {
          real_width = constraints.maxWidth;
          real_height = constraints.maxHeight;
        }
        //????
        return ClipRect(
          child: OverflowBox(
            maxWidth: real_width,
            maxHeight: real_height,
            minHeight: 0,
            minWidth: 0,
            child: nativeMyCameraView,
          ),
          clipper: CustomRect(
            right: constraints.maxWidth,
            bottom: constraints.maxHeight,
          ),
        );
      },
    );
  }

  Future<void> onPlatformViewCreated(int id) async {
    //debugPrint("onPlatformViewCreated");
    //Future.delayed(Duration(seconds: 1),() async {
    debugPrint('onPlatformViewCreated');
      final NativeCameraUint controller =  await NativeCameraUint.init_channel00(
        id,
        this,
      );
      //debugPrint("onCameraCreated");
      if (widget.onCameraCreated != null) {
        await widget.onCameraCreated!(controller, real_width, real_height);
      }
   // });

  }

  int findGcm(int a, int b) {
    return b == 0 ? a : findGcm(b, a % b); // Not bad for one line of code :)
  }

  String asFraction(int a, int b) {
    int gcm = findGcm(a, b);
    return "${(a / gcm)} : ${(b / gcm)}";
  }

  void onImageCaptured_mobile_1(Uint8List? image1, String text, double match_score, double estimate_height, String remark, dynamic params) {
    if (mounted) {
      if (widget.onImageCaptured_mobile_2 != null) {
        widget.onImageCaptured_mobile_2!(image1, text, match_score, estimate_height, remark, params);
      }
    }
  }

  void setZoom(double value) {
    widget.setZoom!(value);
  }
}

class CustomRect extends CustomClipper<Rect> {
  final double right;
  final double bottom;

  CustomRect({this.right = 0, this.bottom = 0});

  @override
  Rect getClip(Size size) {
    Rect rect = Rect.fromLTRB(0.0, 0.0, right, bottom);
    return rect;
  }

  @override
  bool shouldReclip(CustomRect oldClipper) {
    return true;
  }
}

typedef void ImageCandidateCallback(Uint8List imageBytes, String text, double match, String remark, dynamic params);
