//
//  ViewController.m
//  test_mlpr
//
//  Created by ansun on 2023/6/2.
//

#import "ViewController.h"
#include "BitMapPublic.h"
#import "ZipArchive/ZipArchive.h"


#define iMyPixelColorFormat  (kCGImageByteOrder32Little | kCGImageAlphaPremultipliedFirst)



@interface ImageHelper : NSObject {
    
}

/** Converts a UIImage to RGBA8 bitmap.
 @param image - a UIImage to be converted
 @return a RGBA8 bitmap, or NULL if any memory allocation issues. Cleanup memory with free() when done.
 */
+ ( char *) convertUIImageToBitmapRGBA8:(UIImage *)image  rowBytes:(int*)rowBytes; // uiimagetobitmap

/** A helper routine used to convert a RGBA8 to UIImage
 @return a new context that is owned by the caller
 */
+ (CGContextRef) newBitmapRGBA8ContextFromImage:(int)width height:(int)height buffer:(char*)bitmapData;//(CGImageRef)image;


/** Converts a RGBA8 bitmap to a UIImage.
 @param buffer - the RGBA8 unsigned char * bitmap
 @param width - the number of pixels wide
 @param height - the number of pixels tall
 @return a UIImage that is autoreleased or nil if memory allocation issues
 */
+ (UIImage *) convertBitmapRGBA8ToUIImage:(unsigned char *)buffer
                                withWidth:(int)width
                               withHeight:(int)height;

@end



@implementation ImageHelper


+ (char *) convertUIImageToBitmapRGBA8:(UIImage *) image1 rowBytes:(int*)rowBytes
{
    CGSize theSize = image1.size;
    int width = theSize.width;
    int height = theSize.height;
    int bufferLength = width * height * 4;
    char *bitmapData = (char *)malloc(bufferLength);
    CGContextRef context = [self newBitmapRGBA8ContextFromImage:width height:height buffer:bitmapData];
    if(context == 0) {
        free(bitmapData);
        return NULL;
    }
    
    CGRect rect = CGRectMake(0, 0, width, height);
    
    
    if (0) {
        CGContextTranslateCTM(context, 0, height);
        CGContextScaleCTM(context, 1.0, -1.0);
        UIGraphicsPushContext(context);
        [image1 drawInRect:rect]; //memory leak, iOS bug
        UIGraphicsPopContext();
    } else {
        CGImageRef quartzImage = image1.CGImage;
        ////("1.1cfimageref retainCount %d\n",(int)CFGetRetainCount(quartzImage));
        //quartzImage = image1.CGImage;
        //("1.2cfimageref retainCount %d\n",(int)CFGetRetainCount(quartzImage));
        CGContextDrawImage(context,rect, quartzImage); //2021/01/20
        //CGImageRelease(quartzImage);
        //("2cfimageref retainCount %d\n",(int)CFGetRetainCount(quartzImage));
    }
    
    //unsigned char *bitmapData = (unsigned char *)CGBitmapContextGetData(context);
    size_t bytesPerRow = CGBitmapContextGetBytesPerRow(context);
    CGContextRelease(context);
    *rowBytes = (int)bytesPerRow;
    return bitmapData;
}

+ (CGContextRef) newBitmapRGBA8ContextFromImage:(int )width height:(int)height buffer:(char*)bitmapData
{
    CGContextRef context = NULL;
    CGColorSpaceRef colorSpace;
    //uint32_t *bitmapData;
    
    size_t bitsPerPixel = 32;
    size_t bitsPerComponent = 8;
    size_t bytesPerPixel = bitsPerPixel / bitsPerComponent;
    
    //size_t width = CGImageGetWidth(imageRef2);
    //size_t height = CGImageGetHeight(imageRef2);
    
    size_t bytesPerRow = width * bytesPerPixel;
    //size_t bufferLength = bytesPerRow * height;
    
    colorSpace = CGColorSpaceCreateDeviceRGB();
    
    if(!colorSpace) {
        NSLog(@"Error allocating color space RGB\n");
        return NULL;
    }
    
    
    context = CGBitmapContextCreate(bitmapData,
                                    width,
                                    height,
                                    bitsPerComponent,
                                    bytesPerRow,
                                    colorSpace,
                                    iMyPixelColorFormat);    // RGBA
    if(!context) {
        // free(bitmapData);
        printf("error: Bitmap context not created\n");
    }
    
    CGColorSpaceRelease(colorSpace);
    
    return context;
}

+ (UIImage *) convertBitmapRGBA8ToUIImage:(unsigned char *) buffer
                                withWidth:(int) width
                               withHeight:(int) height {
    
    
    size_t bufferLength = width * height * 4;
    CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, buffer, bufferLength, NULL);
    size_t bitsPerComponent = 8;
    size_t bitsPerPixel = 32;
    size_t bytesPerRow = 4 * width;
    
    CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
    if(colorSpaceRef == NULL) {
        printf("error: allocating color space");
        CGDataProviderRelease(provider);
        return nil;
    }
    
    CGBitmapInfo bitmapInfo = iMyPixelColorFormat;
    CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;
    
    CGImageRef iref = CGImageCreate(width,
                                    height,
                                    bitsPerComponent,
                                    bitsPerPixel,
                                    bytesPerRow,
                                    colorSpaceRef,
                                    bitmapInfo,
                                    provider,    // data provider
                                    NULL,        // decode
                                    YES,            // should interpolate
                                    renderingIntent);
    
    uint32_t* pixels = (uint32_t*)malloc(bufferLength);
    
    if(pixels == NULL) {
        NSLog(@"Error: Memory not allocated for bitmap");
        CGDataProviderRelease(provider);
        CGColorSpaceRelease(colorSpaceRef);
        CGImageRelease(iref);
        return nil;
    }
    
    CGContextRef context = CGBitmapContextCreate(pixels,
                                                 width,
                                                 height,
                                                 bitsPerComponent,
                                                 bytesPerRow,
                                                 colorSpaceRef,
                                                 bitmapInfo);
    
    if(context == NULL) {
        NSLog(@"Error context not created");
        free(pixels);
        CGDataProviderRelease(provider);
        CGColorSpaceRelease(colorSpaceRef);
        CGImageRelease(iref);
        return 0;
    }
    
    UIImage *image = nil;
    if(context) {
        
        CGContextDrawImage(context, CGRectMake(0.0f, 0.0f, width, height), iref);
        if (context == 0) {
            printf("2-CGBitmapContextCreate = 0");
        }
        CGImageRef imageRef = CGBitmapContextCreateImage(context);
        
        // Support both iPad 3.2 and iPhone 4 Retina displays with the correct scale
        if([UIImage respondsToSelector:@selector(imageWithCGImage:scale:orientation:)]) {
            float scale = [[UIScreen mainScreen] scale];
            image = [UIImage imageWithCGImage:imageRef scale:scale orientation:UIImageOrientationUp];
        } else {
            image = [UIImage imageWithCGImage:imageRef];
        }
        
        CGImageRelease(imageRef);
        CGContextRelease(context);
    }
    
    CGColorSpaceRelease(colorSpaceRef);
    CGImageRelease(iref);
    CGDataProviderRelease(provider);
    
    if(pixels) {
        free(pixels);
    }
    return image;
}

@end

@interface ViewController ()

@end

@implementation ViewController


- (NSString *)applicationDocumentsDirectory {
    NSURL *url = [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
    NSString *path = url.path;
    return path;
    
}

extern "C" void ocr_call_back(void *userRef, // 用戶自備
                                char *lpr_num_text, //辨識結果車牌文字,例如: 'ABC-1234'
                                float score, //分數 0.0～1.0
                                float estimate_height, // 車牌字母的高度點數pixel，在相同焦距zoom條件下，點數越大代表距離越近。
                                BitMapPublic *lpr_captured_bits // 車牌截圖
                                )
{
    
    UIImage *image = [ImageHelper convertBitmapRGBA8ToUIImage:lpr_captured_bits->buff withWidth:lpr_captured_bits->width withHeight:lpr_captured_bits->height];
    printf("%s score:%.2f 字母高度：%.2f\n",lpr_num_text,score,estimate_height);
    
}


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    NSString *filepath = [[NSBundle mainBundle] pathForResource:@"packed-patttern.bin" ofType:@"zip"];
    ZipArchive *zipArchive = [[ZipArchive alloc] init];
    [zipArchive UnzipOpenFile:filepath Password:@""];
    
    NSString *pathToDirectory = [self applicationDocumentsDirectory];
    NSString *pattern_bath = [pathToDirectory stringByAppendingFormat:@"/packed-patttern.bin"];
    NSFileManager *finder = [NSFileManager defaultManager];
    bool done;
    //必須檢查 packed-patttern.bin 這個檔案是否已經解壓縮？
    if ([finder fileExistsAtPath:pattern_bath] == YES) {
        //已經解壓縮，不可以再加壓縮，否則會出錯。
        done = YES;
    } else {
        //還沒解壓縮，必須解壓縮，否則會出錯。
        printf("%s\n",pathToDirectory.UTF8String);
        done = [zipArchive UnzipFileTo:pathToDirectory overWrite:YES];
        [zipArchive UnzipCloseFile];
        printf("UnzipFileTo done = %d\n",done);
    }
    if (done) {
        /* 測試用序號
         20230608-09363763-01000001-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEGEKEKEHEGEDEHEKEGEDDPEJEKECEDEFEGEEEDEBCMDBDADACMDBCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000002-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEFECEJEHDPEHECEHEBEBEHEFEKEHCMDBDADACMDCCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000003-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEFEFEFEGDPEHEEEFEDEJEFEJEAEGCMDBDADACMDDCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000004-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEFEKEDEBDPEHEHEAEBECEKEJEFEKCMDBDADACMDECMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000005-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEGECEJEBDPEHEKECEFECEHEGEAECCMDBDADACMDFCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000006-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEGEFEBEJDPEJEAEHEDEEEBEDEBEFCMDBDADACMDGCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000007-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEGEHEFEADPEJECEBEFEJEDEJEDEGCMDBDADACMDHCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000008-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEGEKEFEDDPEJEDEDEJEEEGEFEAEJCMDBDADACMDJCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000009-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEHECEEEKDPEJEFEBEEEBEAEFEKEECMDBDADACMDKCMQFLQJHQKMEMMQHLHKBQGJLJA
         20230608-09363763-01000010-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEHEAEAEAEKEFEHEFEFEKDPEJEGEKEEECEFEAEKEHCMDBDADACMDBDACMQFLQJHQKMEMMQHLHKBQGJLJA
         */
        const char *install_key = "20230608-09363763-01000001-debug+HEHFHCNCJFHHDPEBEDEGEFEGEKEKEHEGEDEDEDEEEADPEBEDEGEFEGEKEKEHEGEDEHEKEGEDDPEJEKECEDEFEGEEEDEBCMDBDADACMDBCMQFLQJHQKMEMMQHLHKBQGJLJA";
       
        const char *error = apii_init_ocr_pattern(pattern_bath.UTF8String,install_key);
        if (error) {
            printf("apii_init_ocr_pattern error =  %s\n",error);
        } else {
            UIImage *image = [UIImage imageNamed:@"1.jpg"]; // 車牌樣張
            int rowbytes;
            unsigned char *buff = (unsigned char *)[ImageHelper convertUIImageToBitmapRGBA8:image rowBytes:&rowbytes];
            
            int width = image.size.width;
            int height = image.size.height;
            BitMapPublic bits2;
            
            bits2.buff = buff;
            bits2.rowBytes = rowbytes;
            bits2.width = width;
            bits2.height = height;
            IntRect myClip = {0,0,width,height};
            int miniCharHeightToRecognize = 12; //照片中可以辨識的最小字母高度 pixels。
            /*
            呼叫 apii_DoOCRImage 時指定的 clip 越大則消耗記憶體越多，如果加上 multi-thread，就可能超用記憶體。
             高解析度照片會吃掉大量記憶體，而且辨識非常沒有效率。
             apii_DoOCRImage() 應該包在 nsthread 或 java thread內。
            */
            for (int i=0;i<8;i++) {
                printf("----index %d:\n",i); // apii_DoOCRImage 會自動調用二值化中間值，重複呼叫會使用不同的二值化而得到不同的辨識結果。
                apii_DoOCRImage(bits2,
                                myClip,
                                miniCharHeightToRecognize ,
                                ocr_call_back,
                                (__bridge void*)self
                                );
                
            }
            free(buff);
        }
    }
   
    
}


@end
